package sugg;

import java.io.File;
import java.io.IOException;
import java.util.Map.Entry;

import sugg.matrix.SMatrix;
import sugg.matrix.SVector;
import sugg.objects.Location;
import sugg.tools.Tools;

public class DenseExp {

	public static SMatrix getDenseMatrix (SMatrix queryDocMatrix, Location [] docLocArray, String pathPrefix, double epsilon) throws IOException
	{
		Tools.mkdir(pathPrefix + "dense/");
		String denseFolder = pathPrefix + "dense/ep" + epsilon + "/";
		Tools.mkdir(denseFolder);
		
		String denseMatrixPath = denseFolder + "query_doc_ep_" + epsilon + ".matrix";
		
		SMatrix denseMatrix = null;
		if (new File(denseMatrixPath).exists())
		{
			denseMatrix = new SMatrix(new File(denseMatrixPath), false);
		}
		else
		{
			System.out.println("Building dense matrix ...");
			int numQuery = queryDocMatrix.numRows();
			int numDoc = queryDocMatrix.numColumns();
			
			String docDocMatrixPath = denseFolder + "doc_doc_ep_" + epsilon + ".matrix";
			SMatrix docDocMatrix = null;
			if (new File(docDocMatrixPath).exists())
			{
				docDocMatrix = new SMatrix(new File(docDocMatrixPath), false);
			}
			else
			{
				System.out.println("Building doc-doc matrix ...");
				SMatrix docQueryMatrix = queryDocMatrix.transpose();
				docDocMatrix = new SMatrix(numDoc, numDoc);
				for (int docId1 = 0; docId1 < numDoc; ++docId1)
				{
					if (docId1 % 1000 == 0)
						System.out.printf("\rBuilding doc-doc matrix ... %f%%", ((double)docId1 / numDoc * 100));

					//docId1 < docId2
					for (int docId2 = docId1 + 1; docId2 < numDoc; ++docId2)
					{
						double sim = docSimilarity(queryDocMatrix, docQueryMatrix, docId1, docId2, docLocArray, epsilon);
						if (sim == 0)
							continue;
						docDocMatrix.set(docId1, docId2, sim);
					}
				}
				System.out.println();
				docDocMatrix.write(new File(docDocMatrixPath), false);
			}
			System.out.println("Filling doc-doc matrix ...");
			for (int docId1 = 0; docId1 < numDoc; ++docId1)
			{
				//docId1 < docId2
				for (Entry<Integer, Double> entry : docDocMatrix.getRowVector(docId1))
				{
					int docId2 = entry.getKey();
					if (docId2 > docId1 && docDocMatrix.get(docId1, docId2) > 0)
						docDocMatrix.set(docId2, docId1, docDocMatrix.get(docId1, docId2));
				}
//				for (int docId2 = docId1 + 1; docId2 < numDoc; ++docId2)
//				{
//					if (docDocMatrix.get(docId1, docId2) > 0)
//					{
//						docDocMatrix.set(docId2, docId1, docDocMatrix.get(docId1, docId2));
//					}
//				}
			}

			System.out.println("Filling dense matrix ...");
			denseMatrix = new SMatrix(numQuery, numDoc);
			double oldCount = 0, newCount = 0;
			for (int queryId = 0; queryId < numQuery; ++queryId)
			{
				if (queryId % 1000 == 0)
					System.out.printf("\rFilling dense matrix ... %f %f, %f%%", oldCount, newCount,
							((double)queryId / numQuery * 100));

				SVector simVec = new SVector();
				SVector normVec = new SVector();
				SVector queryDocVec = queryDocMatrix.getRowVector(queryId);
				for (Entry<Integer, Double> entry : queryDocVec)
				{
					int docId = entry.getKey();
					double rating = entry.getValue();
					//System.out.print(rating + " ");
					denseMatrix.set(queryId, docId, rating);
					oldCount++;

					SVector docDocVec = docDocMatrix.getRowVector(docId);
					for (Entry<Integer, Double> simDocEntry : docDocVec)
					{
						int simDocId = simDocEntry.getKey();
						double docSim = simDocEntry.getValue();
						simVec.add(simDocId, rating * docSim);
						normVec.add(simDocId, Math.abs(docSim));
						//System.out.print(simDocId + "," + docSim + " ");
					}
				}
				//System.out.println();
				
				for (Entry<Integer, Double> entry : simVec)
				{
					int docId = entry.getKey();
					if (denseMatrix.contains(queryId, docId))
						continue;
					double sim = entry.getValue();
					double norm = normVec.get(docId);
					
					if (sim > 0 && norm > 0)
					{
						//System.out.println(queryId + " " + docId + ": " + sim + " " + norm + " " + (sim / norm));
						denseMatrix.set(queryId, docId, sim);
						newCount += 1;						
					}
				}
				
//				for (int docId = 0; docId < numDoc; ++docId)
//				{
//					if (queryDocMatrix.get(queryId, docId) == 0)
//					{
//						all++;
//						double sim = 0;
//						double norm = 0;
//
//						SVector docDocVec = docDocMatrix.getRowVector(docId);
//						if (docDocVec.size() > queryDocVec.size())
//						{
//							for (Entry<Integer, Double> entry : queryDocVec)
//							{
//								int otherDocId = entry.getKey();
//								if (docDocVec.contains(otherDocId) == false)
//									continue;
//								double docSim = docDocVec.get(otherDocId);
//								double rating = entry.getValue();
//								sim += (docSim * rating);
//								norm += Math.abs(docSim);
//							}							
//						}
//						else
//						{
//							for (Entry<Integer, Double> entry : docDocVec)
//							{
//								int simDocId = entry.getKey();
//								if (queryDocVec.contains(simDocId) == false)
//									continue;
//								double docSim = entry.getValue();
//								double rating = queryDocVec.get(simDocId);
//								sim += (docSim * rating);
//								norm += Math.abs(docSim);
//							}							
//						}
//
//						if (norm > 0 && sim > 0)
//						{
//							nonzero++;
//							denseMatrix.set(queryId, docId, sim / norm);
//							newCount += 1;
//						}
//					}
//					else
//					{
//						denseMatrix.set(queryId, docId, queryDocMatrix.get(queryId, docId));
//						oldCount += 1;
//					}
//				}
			}
			System.out.println();
			System.out.println("Old Ratio = " + (oldCount / numQuery / numDoc));
			System.out.println("New Ratio = " + ((oldCount + newCount) / numQuery / numDoc));
			denseMatrix.write(new File(denseMatrixPath), false);
		}

		return denseMatrix;
	}
	
	private static double docSimilarity (SMatrix queryDocMatrix, SMatrix docQueryMatrix, int doc1, int doc2, Location [] docLocArray, double epsilon)
	{
		double dist = docLocArray[doc1].distance(docLocArray[doc2]);
		if (dist > epsilon)
			return 0.0;
		
		double sim = 0;
		double norm1 = 0, norm2 = 0;
		SVector vec1 = docQueryMatrix.getRowVector(doc1);
		SVector vec2 = docQueryMatrix.getRowVector(doc2);

		for (Entry<Integer, Double> entry1 : vec1)
		{
			int queryId = entry1.getKey();
			double val1 = entry1.getValue();
			double val2 = vec2.get(queryId);
			double queryAvg = queryDocMatrix.getRowVector(queryId).getAverage();
			norm1 += Math.pow(val1 - queryAvg, 2);

			if (val2 > 0)
			{
				//System.out.println(val1 + "\t" + val2);
				sim += (val1 - queryAvg) * (val2 - queryAvg);
				norm2 += Math.pow(val2 - queryAvg, 2);
			}
			//else skip
		}
		for (Entry<Integer, Double> entry2 : vec2)
		{
			int queryId = entry2.getKey();
			double val2 = entry2.getValue();
			double queryAvg = queryDocMatrix.getRowVector(queryId).getAverage();
			norm2 += Math.pow(val2 - queryAvg, 2);
		}

		if (norm1 == 0 || norm2 == 0 || sim <= 0)
			return 0.0;
		//System.out.println(sim + "/" + (Math.sqrt(norm1) * Math.sqrt(norm2)));
		return sim / (Math.sqrt(norm1) * Math.sqrt(norm2));
	}
}
