package sugg;
import java.io.IOException;

import sugg.tools.AOL;

/*
 * This project is dependent on the following libraries
 * 
 * JWI: for word processing, e.g., stemming
 * http://projects.csail.mit.edu/jwi/
 * 
 * JMathPlot: for intuitive plots showing walked documents, can be removed together with relevant codes
 * https://github.com/yannrichet/jmathplot
 * 
 * Lucene: basic search engine functions
 * https://lucene.apache.org
 * 
 * JDBM: disk resident BTree as index
 * Sources included
 * 
 * */

public class Main {
	/**
	 * Choose one of the following input types:
	 * "twitter_new_york/synthetic"	//test twitter dataset
	 * "twitter_new_york/200000/synthetic"	//test twitter dataset with the first 200K records
	 * "AOL"	//test AOL dataset
	 * "AOL_N"	//test AOL dataset using the first N files, where N is a number from 1 to 10
	 * */
	public static String inputType = "twitter_new_york/synthetic";
	
	/**
	 * Clustering method with format: [DocCluster]_[QueryCluster]
	 * DocCluster methods include: STR, Cell, Rand
	 * QueryCluster methods include: kMeans, kMedoid, Rand
	 */
	public static String method = "Cell_kMeans";//"STR_kMeans";//"Cell_kMedoid";
	
	public static String sourcefile;
	public static String indexfile;
	
	//set num < 0 to use all records
	public static int num = -1;

	//switch to true to force build new Lucene index
	public static final boolean buildLuceneIndex = false;
	//switch to true to force build new inverted index
	public static final boolean buildInvertedIndex = false;
	
	//tf threshold for selecting grams
	public static final int threshold = 3;
	public static final int maxGram = 10;
	
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException
	{
		if (inputType.startsWith("twitter"))
		{
			if (inputType.equals("twitter_new_york/synthetic"))
			{
				sourcefile = "data/twitter/tweets_new_york.txt";
				indexfile = "index/twitter_new_york/tweets.idx";
			}
			else if (inputType.equals("twitter_new_york/200000/synthetic"))
			{
				sourcefile = "data/twitter/tweets_new_york.txt";
				indexfile = "index/twitter_new_york/200000/tweets.idx";
				num = 200000;
			}
		}
		else
		{
			inputType = "AOL";//"AOL_5";	//number of files to consider, 1 - 10			
			AOL aol = new AOL();
			aol.parseData(inputType);
		}
				
		Suggestor suggestor = new Suggestor(inputType);
		suggestor.sAnalyze(method);
		//suggestor.sAnalyze_MoreEdge(method);
		//suggestor.sAnalyzeDense(method, 0.01);
	}
}
