package sugg;

import java.awt.Color;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Scanner;
import java.util.Map.Entry;
import java.util.Set;

import org.math.plot.Plot2DPanel;
import org.math.plot.plots.Plot;
import org.math.plot.plots.ScatterPlot;
import org.math.plot.render.AbstractDrawer;

import sugg.cluster.Clusteror;
import sugg.matrix.SMatrix;
import sugg.matrix.SVector;
import sugg.objects.Location;
import sugg.objects.Pair;
import sugg.objects.Query;
import sugg.objects.ResultStat;
import sugg.objects.Session;
import sugg.plot.PlotTools;
import sugg.tools.Comparators;
import sugg.tools.NodeWeighter;
import sugg.tools.SortedMap;
import sugg.tools.Timer;
import sugg.tools.Tools;
import sugg.tools.VectorWeighter;

public class Suggestor {
	
	final double TRUTH_THRESHOLD = 0.00001;
	
	int numRepeat = 50;
	int k = 5;
	double alpha = 0.5;		//alpha stays
	double beta = 0.5;		// beta * text + (1 - beta) * dist
	double gamma = 1;
	double threshold = 0.00001;
	double range = 1;
	
	boolean test = false;
	
	boolean AOL = false;
	
	public void sAnalyze(String method) throws IOException
	{
		this.preAnalyze();
		
		int [] gramCount = new int[10];
		for (int i = 0; i < numQuery; ++i)
		{
			SVector vec = this.sQueryDocMatrix.getRowVector(i);
			if (vec.numEntries() > 0)
			{
				String query = queryIdToStringArray[i];
				int length = query.split(" ").length;
				if (length <= 10)
					gramCount[length - 1]++;
			}
		}
		for (int i = 0; i < 10; ++i)
		{
			System.out.println("#" + (i + 1) + "-Gram = " + gramCount[i]);
		}
	    
		System.out.println("Testing ... ");

		BCAWalker bca = new BCAWalker(AOL, sQueryDocMatrix, sDocQueryMatrix, sRawQueryDocMatrix, this.numQuery, this.numDoc, 
				this.queryIdToStringArray, this.docToLocArray, this.docToTextArray, this.cellIdToLocArray,
				this.queryStringToClusterMap, this.queryStringToIdMap,
				this.resultPathPrefix);
		
		if (test)
		{
		    //testMento();
			this.testBCA(bca);
			return;
		}
		
		PrintWriter writer = new PrintWriter(new FileWriter(this.outputPathPrefix + "result/stat.txt", true));

		String testFilePath = this.outputPathPrefix + "rand_" + numRepeat + ".txt";
		
		String truthPathPrefix = this.resultPathPrefix + "truth_a" + alpha + "_b" + beta + "_k" + k + "_g" + gamma + "_" + numRepeat;

		//Generate test queries
		range = 1.0;
		this.testBatchBCA(numQuery, numDoc, bca, testFilePath, writer, truthPathPrefix);

		alpha = 0.5;
		beta = 0.5;
		gamma = 1;
		range = 1.0;
		
		int defaultN = this.inputType.contains("AOL") ? 8 : 4;
		
		Query [] randQueryArray = new Query[numRepeat];
		this.generateRandQuery(testFilePath, numRepeat, randQueryArray);
		
		/*
		 * Uncomment to do experiments with DEXA
		 * */
//		this.DEXA(this.k, randQueryArray, 0.1);
//		this.DEXA(this.k, randQueryArray, 0.01);
//		this.DEXA(this.k, randQueryArray, 0.001);
//		this.DEXA(this.k, randQueryArray, 0.0001);
//		if (2 > 1)
//			return;
		
		double [] validRange = {0.02, 0.04, 0.06, 0.08, 0.1};
		int [] numValid = new int[validRange.length];
		for (int i = 0; i < numRepeat; ++i)
		{
			int queryId = randQueryArray[i].queryId;
			Location loc = randQueryArray[i].queryLoc;
			
			SVector vec = this.sQueryDocMatrix.getRowVector(queryId);
			for (Entry<Integer, Double> entry : vec)
			{
				int docId = entry.getKey();
				Location docLoc = docToLocArray[docId];
				double distance = Tools.distance(loc, docLoc) / Tools.SQRT2;

				for (int jj = validRange.length - 1; jj >= 0; --jj)
				{
					//largest to smallest
					if (distance <= validRange[jj])
						numValid[jj]++;
					else
						break;
				}
			}
		}
		
		writer.println("****************************");
		for (int i = 0; i < validRange.length; ++i)
			writer.println("##" + validRange[i] + "\t" + (double)numValid[i]/numRepeat);
		writer.println("****************************");
		writer.flush();
		
		//Generate truth set
		threshold = 0.000009;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		
		threshold = 0.00000001;
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);		
		//this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
//		this.testClusterBCA(4, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
//		this.testClusterBCA(4, 1, "Rand_Rand", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		
		beta = 0.5;
		range = 1.0;
		threshold = 0.00001;
//		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
//		this.testClusterBCA(2, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
//		this.testClusterBCA(2, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
//		this.testClusterBCA(1, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		threshold = 0.00001;
		range = 1.0;

		beta = 0.5;

		beta = 0;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//beta = 0.25;
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		beta = 0.5;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//beta = 0.75;
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		beta = 0.99;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);

		/*	
		beta = 0.5;
		range = 0.8;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.6;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.4;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.2;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.1;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		*/
		
		/*	
		beta = 0.5;
		range = 1.0;
		threshold = 0.00001;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(2, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(2, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(4, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(4, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(2, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 16, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(8, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(4, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 64, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(16, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(16, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 256, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		range = 1.0;
		threshold = 0.0001;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.00005;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.00001;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.000005;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.000001;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		

		threshold = 0.00001;
		alpha = 0.2;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		alpha = 0.35;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		alpha = 0.65;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		alpha = 0.8;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		 */
		
		writer.close();
	}
	
	public void sAnalyze_MoreEdge(String method) throws IOException
	{
		this.preAnalyze();
		
		String docDocMatrixPath = this.outputPathPrefix + "matrix/doc_doc.matrix";
		File docDocMatrixFile = new File(docDocMatrixPath);
		SMatrix sDocDocMatrix = null;
		if (docDocMatrixFile.exists())
		{
			sDocDocMatrix = new SMatrix(docDocMatrixFile, false);
		}
		else
		{
			sDocDocMatrix = new SMatrix(numDoc, numDoc);
			for (int dId1 = 0; dId1 < numDoc; ++dId1)
			{
				if (dId1 % 1000 == 0)
					System.out.println(dId1 + "/" + numDoc);
				SVector docQueryVec1 = this.sDocQueryMatrix.getRowVector(dId1);
				for (int dId2 = dId1 + 1; dId2 < numDoc; ++dId2)
				{
					SVector docQueryVec2 = this.sDocQueryMatrix.getRowVector(dId2);
					double sim = docQueryVec1.cosineSimilarity(docQueryVec2);
					if (sim > 0.01)
					{
						sDocDocMatrix.set(dId1, dId2, sim);
						sDocDocMatrix.set(dId2, dId1, sim);
					}
				}
			}
			for (int dId = 0; dId < numDoc; ++dId)
			{
				SVector docDocVec = sDocDocMatrix.getRowVector(dId);
				if (docDocVec == null || docDocVec.size() == 0)
					continue;
				docDocVec.normalize();
			}
			sDocDocMatrix.write(docDocMatrixFile, false);
		}
		
		System.out.println("Testing ... ");

		BCAWalker bca = new BCAWalker(AOL, sQueryDocMatrix, sDocQueryMatrix, sRawQueryDocMatrix, this.numQuery, this.numDoc, 
				this.queryIdToStringArray, this.docToLocArray, this.docToTextArray, this.cellIdToLocArray,
				this.queryStringToClusterMap, this.queryStringToIdMap,
				this.resultPathPrefix);
		
		PrintWriter writer = new PrintWriter(new FileWriter(this.outputPathPrefix + "result/stat_MoreEdge.txt", true));

		String testFilePath = this.outputPathPrefix + "rand_" + numRepeat + ".txt";
		
		String truthPathPrefix = this.resultPathPrefix + "truth_a" + alpha + "_b" + beta + "_k" + k + "_g" + gamma + "_" + numRepeat;

		//Generate test queries
		range = 1.0;
		this.testBatchBCA(numQuery, numDoc, bca, testFilePath, writer, truthPathPrefix);

		alpha = 0.5;
		beta = 0.5;
		gamma = 1;
		range = 1.0;
		
		int defaultN = this.inputType.contains("AOL") ? 8 : 4;
		
		Query [] randQueryArray = new Query[numRepeat];
		this.generateRandQuery(testFilePath, numRepeat, randQueryArray);
		
		double [] validRange = {0.02, 0.04, 0.06, 0.08, 0.1};
		int [] numValid = new int[validRange.length];
		for (int i = 0; i < numRepeat; ++i)
		{
			int queryId = randQueryArray[i].queryId;
			Location loc = randQueryArray[i].queryLoc;
			
			SVector vec = this.sQueryDocMatrix.getRowVector(queryId);
			for (Entry<Integer, Double> entry : vec)
			{
				int docId = entry.getKey();
				Location docLoc = docToLocArray[docId];
				double distance = Tools.distance(loc, docLoc) / Tools.SQRT2;

				for (int jj = validRange.length - 1; jj >= 0; --jj)
				{
					//largest to smallest
					if (distance <= validRange[jj])
						numValid[jj]++;
					else
						break;
				}
			}
		}
		
		writer.println("****************************");
		for (int i = 0; i < validRange.length; ++i)
			writer.println("##" + validRange[i] + "\t" + (double)numValid[i]/numRepeat);
		writer.println("****************************");
		writer.flush();
		
		beta = 0.5;
		range = 1.0;
		threshold = 0.00001;

		beta = 0;	
		this.testBatchBCA_MoreEdge(numQuery, numDoc, bca, testFilePath, writer, sDocDocMatrix);
		beta = 0.25;
		this.testBatchBCA_MoreEdge(numQuery, numDoc, bca, testFilePath, writer, sDocDocMatrix);
		beta = 0.5;		
		this.testBatchBCA_MoreEdge(numQuery, numDoc, bca, testFilePath, writer, sDocDocMatrix);
		beta = 0.75;
		this.testBatchBCA_MoreEdge(numQuery, numDoc, bca, testFilePath, writer, sDocDocMatrix);
		beta = 0.99;
		this.testBatchBCA_MoreEdge(numQuery, numDoc, bca, testFilePath, writer, sDocDocMatrix);
		
		threshold = 0.00001;
		range = 1.0;

		beta = 0.5;

		beta = 0;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		beta = 0.25;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		beta = 0.5;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		beta = 0.75;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		beta = 0.99;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);

		/*	
		beta = 0.5;
		range = 0.8;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.6;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.4;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.2;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		range = 0.1;
		this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		*/
		
		/*	
		beta = 0.5;
		range = 1.0;
		threshold = 0.00001;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(2, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(2, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(4, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(4, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(2, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 16, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(8, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(4, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 64, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(16, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(16, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 256, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		range = 1.0;
		threshold = 0.0001;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.00005;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.00001;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.000005;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		threshold = 0.000001;
		this.testBatchBCA(1, 1, bca, testFilePath, writer, truthPathPrefix);
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		//this.testClusterBCA(defaultN, 1, "Cell", bca, randQueryIdArray, randQueryLocArray, writer, truthPathPrefix);
		

		threshold = 0.00001;
		alpha = 0.2;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		alpha = 0.35;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		alpha = 0.65;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		alpha = 0.8;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		 */
		
		writer.close();
	}
	
	public void sAnalyzeDense(String method, double epsilon) throws IOException
	{
		String outputPathPrefix = this.outputPathPrefix + "dense/ep" + epsilon + "/";
		Tools.mkdir(outputPathPrefix);
		String resultPathPrefix = outputPathPrefix + "result/";
		Tools.mkdir(resultPathPrefix);
		
		this.clusterPathPrefix = outputPathPrefix + "cluster/";
		Tools.mkdir(this.clusterPathPrefix);
		this.clusterMatrixPathPrefix = outputPathPrefix + "matrix/cluster/";
		Tools.mkdir(this.clusterMatrixPathPrefix);
		
		this.testDenseCF(epsilon);
		
		int [] gramCount = new int[10];
		for (int i = 0; i < numQuery; ++i)
		{
			SVector vec = this.sQueryDocMatrix.getRowVector(i);
			if (vec.numEntries() > 0)
			{
				String query = queryIdToStringArray[i];
				int length = query.split(" ").length;
				if (length <= 10)
					gramCount[length - 1]++;
			}
		}
		for (int i = 0; i < 10; ++i)
		{
			System.out.println("#" + (i + 1) + "-Gram = " + gramCount[i]);
		}
	    
		System.out.println("Testing ... ");

		BCAWalker bca = new BCAWalker(AOL, sQueryDocMatrix, sDocQueryMatrix, sRawQueryDocMatrix, this.numQuery, this.numDoc, 
				this.queryIdToStringArray, this.docToLocArray, this.docToTextArray, this.cellIdToLocArray,
				this.queryStringToClusterMap, this.queryStringToIdMap,
				resultPathPrefix);
		
		PrintWriter writer = new PrintWriter(new FileWriter(resultPathPrefix + "stat.txt", true));

		String testFilePath = outputPathPrefix + "rand_" + numRepeat + ".txt";
		
		String truthPathPrefix = resultPathPrefix + "truth_a" + alpha + "_b" + beta + "_k" + k + "_g" + gamma + "_" + numRepeat;

		//Generate test queries
		range = 1.0;
		this.testBatchBCA(numQuery, numDoc, bca, testFilePath, writer, truthPathPrefix);

		alpha = 0.5;
		beta = 0.5;
		gamma = 1;
		range = 1.0;
		
		Query [] randQueryArray = new Query[numRepeat];
		this.generateRandQuery(testFilePath, numRepeat, randQueryArray);
		
		double [] validRange = {0.02, 0.04, 0.06, 0.08, 0.1};
		int [] numValid = new int[validRange.length];
		for (int i = 0; i < numRepeat; ++i)
		{
			int queryId = randQueryArray[i].queryId;
			Location loc = randQueryArray[i].queryLoc;
			
			SVector vec = this.sQueryDocMatrix.getRowVector(queryId);
			for (Entry<Integer, Double> entry : vec)
			{
				int docId = entry.getKey();
				Location docLoc = docToLocArray[docId];
				double distance = Tools.distance(loc, docLoc) / Tools.SQRT2;

				for (int jj = validRange.length - 1; jj >= 0; --jj)
				{
					//largest to smallest
					if (distance <= validRange[jj])
						numValid[jj]++;
					else
						break;
				}
			}
		}
		
		writer.println("****************************");
		for (int i = 0; i < validRange.length; ++i)
			writer.println("##" + validRange[i] + "\t" + (double)numValid[i]/numRepeat);
		writer.println("****************************");
		writer.flush();
		
		beta = 0.5;
		range = 1.0;
		threshold = 0.00001;
		this.testClusterBCA(1, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(2, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(2, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(4, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(4, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(2, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 16, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(8, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(4, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 64, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		
		this.testClusterBCA(16, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(16, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 4, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 256, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);

		this.testClusterBCA(32, 1, "Rand_Rand", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(32, 1, "Cell", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(8, 16, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);
		this.testClusterBCA(1, 1024, "Cell_kMeans", bca, randQueryArray, writer, truthPathPrefix);

		writer.close();
	}
	
	private int numQuery;
	private int numDoc;
	private long numEdge;
	
	private SMatrix sQueryDocMatrix;
	private SMatrix sDocQueryMatrix;
	private SMatrix sRawQueryDocMatrix;
	
	private String outputPathPrefix;
	private String queryStringFilePath;
	private String docLocationFilePath;
	private String docTextFilePath;
	
	/*
	 * only applicable to AOL
	 * */
	private String querySessionFilePath;
	private String sessionFilePath;
	List<Session> sessionList;
	List<List<Integer>> querySessionList;
	/*
	 * only applicable to AOL
	 * */
	
	private String rawQueryDocMatrixPath;
//	private String queryDocMatrixPath;
//	private String docQueryMatrixPath;
	
	private String clusterPathPrefix;
	private String clusterMatrixPathPrefix;
	private String resultPathPrefix;
	
	String [] queryIdToStringArray;
	Map<String, Integer> queryStringToIdMap;
	Map<String, List<Integer>> queryStringToClusterMap;
	Location [] docToLocArray;
	String [] docToTextArray;
	Location [] cellIdToLocArray;
	
	private String inputType;

	public Suggestor(String inputType) throws IOException
	{
		this.inputType = inputType;
		
		this.AOL = this.inputType.contains("AOL");
		System.out.println("AOL? " + this.AOL);

		this.outputPathPrefix = "output/" + inputType + "/";
		Tools.mkdir(this.outputPathPrefix);
		String infoFilePath = this.outputPathPrefix + "info.txt";
		
		this.queryStringFilePath = this.outputPathPrefix + "querystring.txt";
		this.docLocationFilePath = this.outputPathPrefix + "docloc.txt";
		this.docTextFilePath = this.outputPathPrefix + "doctext.txt";
		
		this.querySessionFilePath = this.outputPathPrefix + "querysession.txt";
		this.sessionFilePath = this.outputPathPrefix + "session.txt";
		
		Tools.mkdir(this.outputPathPrefix + "matrix/");
		//this.queryDocMatrixPath = this.outputPathPrefix + "matrix/query_doc.matrix";
		//this.docQueryMatrixPath = this.outputPathPrefix + "matrix/doc_query.matrix";
		this.rawQueryDocMatrixPath = this.outputPathPrefix + "matrix/query_doc.raw.matrix";
		
		this.clusterPathPrefix = this.outputPathPrefix + "cluster/";
		Tools.mkdir(this.clusterPathPrefix);
		this.clusterMatrixPathPrefix = this.outputPathPrefix + "matrix/cluster/";
		Tools.mkdir(this.clusterMatrixPathPrefix);
		this.resultPathPrefix = this.outputPathPrefix + "result/";
		Tools.mkdir(this.resultPathPrefix);
		
		int N = 10;
		
		if (new File(infoFilePath).isFile())
		{
			Scanner scanner = new Scanner(new File(infoFilePath));
			String line = scanner.nextLine();
			this.numQuery = Integer.parseInt(line.split("=")[1].trim());
			line = scanner.nextLine();
			this.numEdge = Integer.parseInt(line.split("=")[1].trim());
			line = scanner.nextLine();
			this.numDoc = Integer.parseInt(line.split("=")[1].trim());
			scanner.close();
		}
		else
		{
			DataGenerator generator = new DataGenerator(inputType);
			generator.generate(N);
			
			this.numQuery = generator.numQuery;
			this.numEdge = generator.numEdge;
			this.numDoc = generator.numDoc;
		}
		
		System.out.println("numQuery = " + numQuery);
		System.out.println("numEdge = " + numEdge);
		System.out.println("numDoc = " + numDoc);

		double interval = 1.0 / N;
		this.cellIdToLocArray = new Location[N * N];
		for (int cellId = 0; cellId != N * N; ++cellId)
		{
			this.cellIdToLocArray[cellId] = Tools.cellIdToLoc(cellId, N, interval);
		}
	}
	
	public void testDenseCF (double epsilon) throws IOException
	{
		this.queryIdToStringArray = new String[this.numQuery];
		this.queryStringToIdMap = new HashMap<String, Integer>();
		this.queryStringToClusterMap = new HashMap<String, List<Integer>>();
		this.docToLocArray = new Location[this.numDoc];

		System.out.println("Reading query strings in " + this.queryStringFilePath);
		Scanner scanner = new Scanner(new File(this.queryStringFilePath));
		for (int i = 0; i != this.numQuery; ++i)
		{
			String query = scanner.nextLine();
			String cellsString = scanner.nextLine().trim();

			List<Integer> clusterList = new ArrayList<Integer>();
			for (String cell : cellsString.split(" "))
			{
				int cellId = Integer.parseInt(cell);
				clusterList.add(cellId);
			}
			this.queryStringToClusterMap.put(query, clusterList);
			
			this.queryIdToStringArray[i] = query;
			this.queryStringToIdMap.put(query, i);
		}
		scanner.close();

		System.out.println("Reading document locations in " + this.docLocationFilePath);
		scanner = new Scanner(new File(this.docLocationFilePath));
		for (int i = 0; i != this.numDoc; ++i)
		{
			double locx = scanner.nextDouble();
			double locy = scanner.nextDouble();
			this.docToLocArray[i] = new Location(locx, locy);
		}
		scanner.close();
		
		if (new File(this.docTextFilePath).isFile())
		{
			this.docToTextArray = new String[this.numDoc];
			System.out.println("Reading document text in " + this.docTextFilePath);
			scanner = new Scanner(new File(this.docTextFilePath));
			for (int i = 0; i != this.numDoc; ++i)
				this.docToTextArray[i] = scanner.nextLine();
			scanner.close();
		}
		
		Tools.mkdir(outputPathPrefix + "dense/");
		String denseFolder = outputPathPrefix + "dense/ep" + epsilon + "/";
		Tools.mkdir(denseFolder);
		String denseMatrixPath = denseFolder + "query_doc_ep_" + epsilon + ".matrix";
				
		if (new File(denseMatrixPath).exists())
		{
			this.sRawQueryDocMatrix = new SMatrix(new File(denseMatrixPath), false);
		}
		else
		{
			System.out.println("Reading Raw-Query-Doc-Matrix in " + this.rawQueryDocMatrixPath);
			this.sRawQueryDocMatrix = new SMatrix(new File(this.rawQueryDocMatrixPath), false);
		    if (this.sRawQueryDocMatrix.numRows() != this.numQuery || this.sRawQueryDocMatrix.numColumns() != this.numDoc)
		    	Tools.printError("Unmatched sizes in RawQueryDocMatrix");
		    
		    // normalize tf-idf scores
		    this.sQueryDocMatrix = new SMatrix(this.numQuery, this.numDoc);
		    for (int i = 0; i != numQuery; ++i)
		    {
		    	SVector rawQueryDocVector = this.sRawQueryDocMatrix.getRowVector(i);

			    double maxScore = -Double.MAX_VALUE;
		    	for (Entry<Integer, Double> docEntry : rawQueryDocVector)
		    	{
		    		double score = docEntry.getValue();
		    		if (score > maxScore)
		    			maxScore = score;
		    	}
		    	for (Entry<Integer, Double> docEntry : rawQueryDocVector)
		    	{
		    		int dId = docEntry.getKey();
		    		double score = docEntry.getValue();
		    		if (this.inputType.contains("AOL") || Math.pow(score/maxScore, 1) > 0.35)
		    		{
		    			this.sQueryDocMatrix.set(i, dId, Math.log10(score + 1));
		    		}
		    	}
		    }
		    
//		    for (int i = 0; i != numQuery; ++i)
//		    	this.sQueryDocMatrix.getRowVector(i).normalize01();
		    
		    this.sRawQueryDocMatrix = DenseExp.getDenseMatrix(this.sQueryDocMatrix, this.docToLocArray, 
		    		this.outputPathPrefix, epsilon);
		}
	    
	    int numEdge = 0;
	    PrintWriter writer = new PrintWriter(new File(this.outputPathPrefix + "edgeWeight.txt"));
	    this.sQueryDocMatrix = new SMatrix(this.numQuery, this.numDoc);
	    for (int i = 0; i != numQuery; ++i)
	    {
	    	SVector rawQueryDocVector = this.sRawQueryDocMatrix.getRowVector(i);
	    	for (Entry<Integer, Double> docEntry : rawQueryDocVector)
	    	{
	    		int dId = docEntry.getKey();
	    		double score = docEntry.getValue();
	    		writer.println(score);
	    		numEdge++;
	    		this.sQueryDocMatrix.set(i, dId, score);
	    	}
	    }
	    writer.close();
	    System.out.println("numEdge = " + numEdge);

	    this.sDocQueryMatrix = this.sQueryDocMatrix.transpose();

	    for (int i = 0; i != numQuery; ++i)
	    	this.sQueryDocMatrix.getRowVector(i).normalize01();
	    
	    for (int i = 0; i != numDoc; ++i)
	    	this.sDocQueryMatrix.getRowVector(i).normalize01();
	    
	    writer = new PrintWriter(new File(this.outputPathPrefix + "queryDegree.txt"));
	    for (int i = 0; i != numQuery; ++i)
	    	writer.println(sQueryDocMatrix.getRowVector(i).numEntries());
	    writer.close();

	    writer = new PrintWriter(new File(this.outputPathPrefix + "docDegree.txt"));
	    for (int i = 0; i != numDoc; ++i)
	    	writer.println(sDocQueryMatrix.getRowVector(i).numEntries());
	    writer.close();
	    
		if (this.AOL)
		{
			this.sessionList = new ArrayList<Session>();
			System.out.println("Reading sessions in " + this.sessionFilePath);
			scanner = new Scanner(new File(this.sessionFilePath));
			int numSession = scanner.nextInt();
			for (int sId = 0; sId < numSession; ++sId)
			{
				int sessionId = scanner.nextInt();
				assert(sId == sessionId);
				
				int lastQueryId = scanner.nextInt();
				int lastDocId = scanner.nextInt();
				Session session = new Session(sessionId, lastQueryId, lastDocId);

				this.sessionList.add(session);
			}
			scanner.close();
			
			this.querySessionList = new ArrayList<List<Integer>>();
			System.out.println("Reading query-session information in " + this.querySessionFilePath);
			scanner = new Scanner(new File(this.querySessionFilePath));
			for (int queryId = 0; queryId < numQuery; ++queryId)
			{
				List<Integer> list = new ArrayList<Integer>();
				int num = scanner.nextInt();
				for (int ii = 0; ii < num; ++ii)
				{
					int sId = scanner.nextInt();
					list.add(sId);
				}
				this.querySessionList.add(list);
			}
			scanner.close();
		}
	}
	
	private void preAnalyze() throws IOException
	{
		this.queryIdToStringArray = new String[this.numQuery];
		this.queryStringToIdMap = new HashMap<String, Integer>();
		this.queryStringToClusterMap = new HashMap<String, List<Integer>>();
		this.docToLocArray = new Location[this.numDoc];

		System.out.println("Reading query strings in " + this.queryStringFilePath);
		Scanner scanner = new Scanner(new File(this.queryStringFilePath));
		for (int i = 0; i != this.numQuery; ++i)
		{
			String query = scanner.nextLine();
			String cellsString = scanner.nextLine().trim();

			List<Integer> clusterList = new ArrayList<Integer>();
			for (String cell : cellsString.split(" "))
			{
				int cellId = Integer.parseInt(cell);
				clusterList.add(cellId);
			}
			this.queryStringToClusterMap.put(query, clusterList);
			
			this.queryIdToStringArray[i] = query;
			this.queryStringToIdMap.put(query, i);
		}
		scanner.close();

		System.out.println("Reading document locations in " + this.docLocationFilePath);
		scanner = new Scanner(new File(this.docLocationFilePath));
		for (int i = 0; i != this.numDoc; ++i)
		{
			double locx = scanner.nextDouble();
			double locy = scanner.nextDouble();
			this.docToLocArray[i] = new Location(locx, locy);
		}
		scanner.close();
		
		if (this.AOL)
		{
			this.sessionList = new ArrayList<Session>();
			System.out.println("Reading sessions in " + this.sessionFilePath);
			scanner = new Scanner(new File(this.sessionFilePath));
			int numSession = scanner.nextInt();
			for (int sId = 0; sId < numSession; ++sId)
			{
				int sessionId = scanner.nextInt();
				assert(sId == sessionId);
				
				int lastQueryId = scanner.nextInt();
				int lastDocId = scanner.nextInt();
				Session session = new Session(sessionId, lastQueryId, lastDocId);

				this.sessionList.add(session);
			}
			scanner.close();
			
			this.querySessionList = new ArrayList<List<Integer>>();
			System.out.println("Reading query-session information in " + this.querySessionFilePath);
			scanner = new Scanner(new File(this.querySessionFilePath));
			for (int queryId = 0; queryId < numQuery; ++queryId)
			{
				List<Integer> list = new ArrayList<Integer>();
				int num = scanner.nextInt();
				for (int ii = 0; ii < num; ++ii)
				{
					int sId = scanner.nextInt();
					list.add(sId);
				}
				this.querySessionList.add(list);
			}
			scanner.close();
		}
		
		if (new File(this.docTextFilePath).isFile())
		{
			this.docToTextArray = new String[this.numDoc];
			System.out.println("Reading document text in " + this.docTextFilePath);
			scanner = new Scanner(new File(this.docTextFilePath));
			for (int i = 0; i != this.numDoc; ++i)
				this.docToTextArray[i] = scanner.nextLine();
			scanner.close();
		}
	    
		System.out.println("Reading Raw-Query-Doc-Matrix in " + this.rawQueryDocMatrixPath);
		this.sRawQueryDocMatrix = new SMatrix(new File(this.rawQueryDocMatrixPath), false);
	    if (this.sRawQueryDocMatrix.numRows() != this.numQuery || this.sRawQueryDocMatrix.numColumns() != this.numDoc)
	    	Tools.printError("Unmatched sizes in RawQueryDocMatrix");

	    int numEdge = 0;
	    
	    PrintWriter writer = new PrintWriter(new File(this.outputPathPrefix + "edgeWeight.txt"));
	    
	    // normalize tf-idf scores
	    this.sQueryDocMatrix = new SMatrix(this.numQuery, this.numDoc);
	    for (int i = 0; i != numQuery; ++i)
	    {
	    	SVector rawQueryDocVector = this.sRawQueryDocMatrix.getRowVector(i);

		    double maxScore = -Double.MAX_VALUE;
	    	for (Entry<Integer, Double> docEntry : rawQueryDocVector)
	    	{
	    		double score = docEntry.getValue();
	    		if (score > maxScore)
	    			maxScore = score;
	    	}
	    	for (Entry<Integer, Double> docEntry : rawQueryDocVector)
	    	{
	    		int dId = docEntry.getKey();
	    		double score = docEntry.getValue();
	    		//score += score * (Math.random() * 0.1 - 0.05);
	    		if (this.inputType.contains("AOL") || Math.pow(score/maxScore, 1) > 0.3)
	    		{
	    			writer.println(score);
	    			numEdge++;
	    			this.sQueryDocMatrix.set(i, dId, Math.log10(score + 1));
	    		}
	    	}
	    }
	    writer.close();
	    System.out.println("numEdge = " + numEdge);

	    this.sDocQueryMatrix = this.sQueryDocMatrix.transpose();

	    for (int i = 0; i != numQuery; ++i)
	    	this.sQueryDocMatrix.getRowVector(i).normalize01();
	    
	    for (int i = 0; i != numDoc; ++i)
	    	this.sDocQueryMatrix.getRowVector(i).normalize01();
	    
	    writer = new PrintWriter(new File(this.outputPathPrefix + "queryDegree.txt"));
	    for (int i = 0; i != numQuery; ++i)
	    	writer.println(sQueryDocMatrix.getRowVector(i).numEntries());
	    writer.close();

	    writer = new PrintWriter(new File(this.outputPathPrefix + "docDegree.txt"));
	    for (int i = 0; i != numDoc; ++i)
	    	writer.println(sDocQueryMatrix.getRowVector(i).numEntries());
	    writer.close();
	    
//	    int queryId = this.queryStringToIdMap.get("kennedy international");
//	    Location queryLoc = new Location(0.3, 0.45);
//	    SVector vec = this.sQueryDocMatrix.getRowVector(queryId);
//	    writer = new PrintWriter(new File("test.txt"));
//	    for (Entry<Integer, Double> entry : vec)
//	    {
//	    	int docId = entry.getKey();
//	    	Location loc = this.docToLocArray[docId];
//	    	if (Tools.distance(queryLoc, loc) <= 0.05)
//	    	{
//	    		writer.println(loc.getLong() + "\t" + loc.getLat());
//	    	}
//	    }
//	    writer.close();	    
	}
	
	public void testMento()
	{
		Scanner in = new Scanner(System.in);
		while(true)
		{
			System.out.print("Input a phrase: ");
			
			String phrase = in.nextLine();
			if (phrase.equals("\n") || phrase.equals(""))
			{
				System.out.println();
				continue;
			}
			if (phrase.equals("$"))
				break;

			if (queryStringToIdMap.containsKey(phrase) == false)
			{
				System.out.println("Choose another phrase .. ");
				continue;
			}
			
			int inputId = queryStringToIdMap.get(phrase);

			System.out.print("Input a location: ");
			double queryLong = in.nextDouble();
			double queryLat = in.nextDouble();
			Location queryLoc = new Location(queryLat, queryLong);
			
			System.out.print("Input NN: ");
			int NN = in.nextInt();
			
			this.mento(inputId, queryLoc, NN);
		}
		in.close();
	}
	
	public void mento(int inputId, Location queryLoc, int NN)
	{
		Timer timer = new Timer();
		
		timer.start();
		
		NodeWeighter weighter = new NodeWeighter(numQuery, numDoc, queryLoc, docToLocArray, gamma, sQueryDocMatrix);
		
		Map<Integer, SVector> queryDocVectorMap = new HashMap<Integer, SVector>();
		Map<Integer, SVector> docQueryVectorMap = new HashMap<Integer, SVector>();

		SortedMap<Integer, Double> sortedResult = new SortedMap<Integer, Double>(new Comparators.CompareDoubleDesc());
		
		for (int nn = 0; nn < NN; ++nn)
		{
			int qId = inputId;
			while(true)
			{
				SVector queryDocVector = VectorWeighter.sGetQueryDocVector(qId, 
						sQueryDocMatrix, weighter, beta, range, queryDocVectorMap);
				
				int dId = 0;
				for (Entry<Integer, Double> docEntry : queryDocVector)
				{
					int docId = docEntry.getKey();
					double val = docEntry.getValue();

					dId = docId;
					if (Math.random() <= val)
						break;
				}
				
				SVector docQueryVector = VectorWeighter.sGetDocQueryVector(dId, 
						sDocQueryMatrix, weighter, beta, range, docQueryVectorMap);
				for (Entry<Integer, Double> queryEntry : docQueryVector)
				{
					int queryId = queryEntry.getKey();
					double val = queryEntry.getValue();
					
					qId = queryId;
					if (Math.random() <= val)
					{
						double num = sortedResult.get(qId) == null ? 0 : sortedResult.get(qId);
						sortedResult.put(qId, num + 1, true);
						break;
					}
				}
				
				if (Math.random() <= alpha)
					break;
			}
		}
		
		sortedResult.sort();
		
		for (int i = 0; i < Math.min(sortedResult.size(), k + 1); ++i)
		{
			int phraseId = sortedResult.getKey(i);
			double score = sortedResult.getValue(i);
			String phrase = queryIdToStringArray[phraseId];
			
			System.out.printf("[%d] %s: %.3f\n", i, phrase, score);
		}
		
		System.out.println("Cost = " + timer.stop());
	}
	
	public void testTranBCA(int queryBatch, BCAWalker bca, 
			Query [] randQueryArray,
			PrintWriter writer, String truthPathPrefix) throws IOException
	{
		writer.println("***************************");
		writer.println("alpha = " + alpha + " epsilon = " + threshold + " gamma = " + gamma + " range = " + range);
		writer.println("TRAN: " + queryBatch);
		
		String truthPath = truthPathPrefix + "_TRAN.txt";
		String resultPath = this.resultPathPrefix + "test_TRAN_" + queryBatch + "_g" + gamma + "_" + range + ".txt";
		
		Scanner truthScanner = null;
		if (range < 1)
			truthScanner = new Scanner(new File(truthPath));
		
		ResultStat finalStat = new ResultStat();
		System.out.println("Start testing with " + numRepeat + " queries, range = " + range);
		for (int i = 0; i != numRepeat; ++i)
		{
			Query query = randQueryArray[i];
			
			List<Integer> truthList = new ArrayList<Integer>();
			if (range < 1)
			{
				String truthLine = truthScanner.nextLine().trim();
				String [] truths = truthLine.split(" ");
				for (String truth : truths)
					truthList.add(Integer.parseInt(truth));
			}
			
			ResultStat stat = bca.tranBCA(query, alpha, beta, gamma, threshold, range,
					queryBatch, resultPath, truthList);
			
			if (range == 1)
			{
				PrintWriter truthWriter = new PrintWriter(new FileWriter(truthPath, true));
				for (Integer truth : truthList)
					truthWriter.print(truth + " ");
				truthWriter.println();
				truthWriter.close();
			}
			
			finalStat.addStat(stat);
			System.out.println(i + ": " + stat.get(0));
		}
		finalStat.avg();
		System.out.println("##Average Cost = " + finalStat.get(0));
		writer.println("##Cost\t" + finalStat.get(0));
		writer.println("##TextScore\t" + finalStat.get(2));
		writer.println("##SimToTruth\t" + finalStat.get(3));
		writer.println("##MisMatch\t" + finalStat.get(4));
		writer.println("##DistScore\t" + finalStat.get(5));
		writer.println("##Iter\t" + finalStat.get(1));
		writer.println("##InputText\t" + finalStat.get(6));
		writer.println("##InputDist\t" + finalStat.get(7));

		if (range < 1)
			truthScanner.close();
	}
	
	public void testBCA(BCAWalker bca) throws IOException
	{

		String plotPathPrefix = this.outputPathPrefix + "plot/";
		Tools.mkdir(plotPathPrefix);
		
		double [][] allLoc = new double[this.numDoc][2];
		for (int i = 0; i < this.numDoc; i += 10)
		{
			Location loc = docToLocArray[i];
			allLoc[i][0] = loc.getLong();
			allLoc[i][1] = loc.getLat();
		}
		
		String allDocLocPath = plotPathPrefix + "docloc.txt";
		PrintWriter writer = new PrintWriter(new File(allDocLocPath));
		for (int i = 0; i < this.numDoc; i += 10)
		{
			Location loc = this.docToLocArray[i];
			writer.println(loc.getLong() + "\t" + loc.getLat());
		}
		writer.close();
		
		Plot allDocPlot = new ScatterPlot("Related Docs", Color.GRAY, AbstractDrawer.ROUND_DOT, 3, allLoc);
		Plot2DPanel allPlotPanel = new Plot2DPanel();
		allPlotPanel.addPlot(allDocPlot);
		allPlotPanel.setFixedBounds(0, 0, 1);
		allPlotPanel.setFixedBounds(1, 0, 1);
		PlotTools.drawPlot(allPlotPanel, "All Docs");
		
		//plotPanel.addPlot(allDocPlot);
		
		int k = 1;
		double beta = 0.5;
		double threshold = 0.00001;
		
		int queryId;
		Location queryLoc;
		
//		queryId = queryStringToIdMap.get("airport");
//		queryLoc = new Location(0.4, 0.2);	//newark
//		beta = 0.5;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		beta = 0;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		beta = 1;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		
//
//		queryLoc = new Location(0.3, 0.45);	//kennedy
//		beta = 0.5;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		beta = 0;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		beta = 1;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//
//
//		queryLoc = new Location(0.5, 0.4);
//		beta = 0.5;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		beta = 0;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
//		beta = 1;
//		bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);

//		Clusteror clusteror = new Clusteror(this.numQuery, this.numDoc, sQueryDocMatrix, sDocQueryMatrix, sRawQueryDocMatrix,
//				this.docToLocArray, this.clusterPathPrefix);
//		int NN = 4; int MM = 1;
//		String clusterMethod = "Cell";
//		int [] docClusterArray = clusteror.getDocClusterArray(NN, MM, clusterMethod);
//		int [] queryClusterArray = clusteror.getQueryClusterArray(NN, MM, docClusterArray, clusterMethod);
//		int numCluster = NN * NN * MM;

		Scanner in = new Scanner(System.in);
		while(true)
		{
			System.out.print("Input a phrase: ");
			
			String phrase = in.nextLine();
			if (phrase.equals("\n") || phrase.equals(""))
			{
				System.out.println();
				continue;
			}
			if (phrase.equals("$"))
				break;

			if (queryStringToIdMap.containsKey(phrase) == false)
			{
				System.out.println("Choose another phrase .. ");
				continue;
			}
			
			queryId = queryStringToIdMap.get(phrase);
			
			Set<Integer> querySet = new HashSet<Integer>();
			for (Entry<String, Integer> entry : queryStringToIdMap.entrySet())
			{
				String query = entry.getKey();
				int id = entry.getValue();
				if (query.contains(phrase))
					querySet.add(id);
			}
			querySet.add(queryId);
			
			Set<Integer> relatedDoc = new HashSet<Integer>();
			for (Integer id : querySet)
			{
				SVector vec = this.sQueryDocMatrix.getRowVector(id);
				for (Entry<Integer, Double> entry : vec)
					relatedDoc.add(entry.getKey());
			}
			double [][] relatedLoc = new double[relatedDoc.size()][2];
			int ii = 0;
			for (Integer docId : relatedDoc)
			{
				Location loc = docToLocArray[docId];
				relatedLoc[ii][0] = loc.getLong();
				relatedLoc[ii][1] = loc.getLat();
				ii++;
			}
			Plot relatedDocPlot = new ScatterPlot("Related Docs", Color.MAGENTA, AbstractDrawer.ROUND_DOT, 3, relatedLoc);
			Plot2DPanel plotPanel = new Plot2DPanel();
			if (relatedDoc.size() > 0)
				plotPanel.addPlot(relatedDocPlot);
			plotPanel.setFixedBounds(0, 0, 1);
			plotPanel.setFixedBounds(1, 0, 1);
			PlotTools.drawPlot(plotPanel, "Related Docs " + phrase);

			System.out.print("Input a location: ");
			double queryLong = in.nextDouble();
			double queryLat = in.nextDouble();
			queryLoc = new Location(queryLat, queryLong);
			
			System.out.print("Input k, beta and epsilon: ");
			k = in.nextInt();
			beta = in.nextDouble();
			threshold = in.nextDouble();
//			k = 5;
//			beta = 0.5;
//			threshold = 0.00001;
			
			bca.testBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, plotPathPrefix);
			//bca.testClusterBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, docClusterArray, queryClusterArray, numCluster, numCluster);
		}
		in.close();
	}
	
	public void testBatchBCA(int queryBatch, int docBatch, BCAWalker bca, 
			String testFilePath,
			PrintWriter writer, String truthPathPrefix) throws IOException
	{	
		File testFile = new File(testFilePath);
		boolean hasTestFile = testFile.isFile();
		Query [] queryArray = new Query[numRepeat];
		if (hasTestFile)
			this.generateRandQuery(testFilePath, numRepeat, queryArray);
//		else
//		{
//			queryArray[0] = new Query();
//			queryArray[1] = new Query();
//			queryArray[0].queryId = this.queryStringToIdMap.get("airport");
//			queryArray[1].queryId = this.queryStringToIdMap.get("airport");
//			queryArray[0].queryLoc = new Location(0.5, 0.4);
//			queryArray[1].queryLoc = new Location(0.3, 0.45);
//		}
		
		writer.println("***************************");
		writer.println("alpha = " + alpha + " epsilon = " + threshold);
		writer.println("beta = " + beta + " gamma = " + gamma + " range = " + range);
		writer.println("BATCH: " + queryBatch + "\t" + docBatch);
		String truthPath = truthPathPrefix + "_BATCH.txt";
		String resultPath = this.resultPathPrefix + "test_TOP_" + queryBatch + "_" + docBatch
				+ "_g" + gamma + "_" + range + ".txt";

		Scanner truthScanner = null;
		if (range < 1 && new File(truthPath).isFile())
			truthScanner = new Scanner(new File(truthPath));
		
		ResultStat finalStat = new ResultStat();
		System.out.println("Start testing with " + numRepeat + " queries, range = " + range);
		for (int i = 0; i != numRepeat; ++i)
		{
			
			List<Integer> truthList = new ArrayList<Integer>();
			if (range < 1 && new File(truthPath).isFile())
			{
				String truthLine = truthScanner.nextLine().trim();
				String [] truths = truthLine.split(" ");
				for (String truth : truths)
					truthList.add(Integer.parseInt(truth));
			}
			
			ResultStat stat = null;
			if (hasTestFile)
			{
				stat = bca.batchBCA(queryArray[i],
						k, alpha, beta, gamma, threshold, range, queryBatch, docBatch, resultPath, truthList);
//				stat = bca.batchBCA(queryId, queryLoc, k, alpha, beta, gamma, threshold, range, resultPath, truthSet);
			}
			else
			{
//				if (i == 0 || i == 1)
//				{
//					stat = bca.batchBCA(queryArray[i],
//						k, alpha, beta, gamma, threshold, range, queryBatch, docBatch, resultPath, truthList);			
//				}
//				else
				{
					while (true)
					{
						List<Integer> clusterList = null;
						queryArray[i] = new Query();
						
						if (AOL)
						{
							while (true)
							{
								int randQueryId = (int) (Math.random() * this.numQuery);
								List<Integer> sessionList = this.querySessionList.get(randQueryId);
								int sessionId = sessionList.get((int) (Math.random() * sessionList.size()));
								
								Session session = this.sessionList.get(sessionId);
								if (session.lastQueryId == randQueryId)
									continue;
								
								String phrase = this.queryIdToStringArray[randQueryId];
								if (phrase.split(" ").length > 3)
									continue;
								clusterList = this.queryStringToClusterMap.get(phrase);
								if (clusterList.size() < 5)
									continue;
								
								int expQueryId = session.lastQueryId;
								int expDocId = session.lastDocId;
								Location sessionLoc = this.docToLocArray[session.lastDocId];

								queryArray[i].queryId = randQueryId;
								queryArray[i].expectedQueryId = expQueryId;
								queryArray[i].expectedDocId = expDocId;
								queryArray[i].queryLoc = sessionLoc;
								break;
							}
						}
						else
						{
							while (true)
							{						
								queryArray[i].queryId = (int) (Math.random() * this.numQuery);
//								if (this.sRawQueryDocMatrix.getRowVector(queryArray[i].queryId).normL1() < 5)
//									continue;
								
								String phrase = this.queryIdToStringArray[queryArray[i].queryId];
								if (phrase.split(" ").length > 1)
									continue;
								clusterList = this.queryStringToClusterMap.get(phrase);
								if (clusterList.size() >= 5)
									break;								
							}
							queryArray[i].queryLoc = new Location(Math.random(), Math.random());
							int cellId = clusterList.get((int)(Math.random() * clusterList.size()));
							queryArray[i].queryLoc = Tools.cellIdToLoc(cellId, 10, 0.1);
						}
						
						System.out.println(queryArray[i].toString());
						
						stat = bca.batchBCA(queryArray[i],
								k, alpha, beta, gamma, threshold, range, queryBatch, docBatch, resultPath, truthList);
						if (stat != null)
						{
							System.out.println(stat.get(11));
							if (stat.get(11) >= 5 && stat.get(11) <= 1000)
								break;
						}
					}
				}
			}
			
			if (range == 1)
			{
				PrintWriter truthWriter = new PrintWriter(new FileWriter(truthPath, true));
				for (Integer truth : truthList)
					truthWriter.print(truth + " ");
				truthWriter.println();
				truthWriter.close();
			}
			
			finalStat.addStat(stat);
			System.out.println(i + ": " + stat.get(0));
		}
		finalStat.avg();
		System.out.println("##Average Cost = " + finalStat.get(0));
		writer.println("##TextScore\t" + finalStat.get(2));	//"##TextScore\t"
		writer.println("##SimToTruth\t" + finalStat.get(3));
		writer.println("##MisMatch\t" + finalStat.get(4));
		writer.println("##DistScore\t" + finalStat.get(5));	//"##DistScore\t"
		writer.println("##Cost\t" + finalStat.get(0));	//"##Cost\t"
		writer.println("##Iter\t" + finalStat.get(1));
		writer.flush();
		
		if (hasTestFile == false)
		{
			PrintWriter testFileWriter = new PrintWriter(testFile);
			for (int i = 0; i != numRepeat; ++i)
			{
				testFileWriter.print(queryArray[i].queryId + "\t");
				if (this.AOL)
				{
					testFileWriter.print(queryArray[i].expectedQueryId + "\t");
					testFileWriter.print(queryArray[i].expectedDocId + "\t");
				}
				testFileWriter.print(queryArray[i].queryLoc.getLocx() + "\t");
				testFileWriter.println(queryArray[i].queryLoc.getLocy());
			}
			testFileWriter.close();
		}

		if (range < 1 && new File(truthPath).isFile())
			truthScanner.close();
	}
	
	public void testBatchBCA_MoreEdge(int queryBatch, int docBatch, BCAWalker bca, 
			String testFilePath, PrintWriter writer, SMatrix sDocDocMatrix) throws IOException
	{	
		Query [] queryArray = new Query[numRepeat];
		this.generateRandQuery(testFilePath, numRepeat, queryArray);
		
		writer.println("***************************");
		writer.println("alpha = " + alpha + " epsilon = " + threshold);
		writer.println("beta = " + beta + " gamma = " + gamma + " range = " + range);
		writer.println("BATCH: " + queryBatch + "\t" + docBatch);
		String resultPath = this.resultPathPrefix + "test_MoreEdge_TOP_" + queryBatch + "_" + docBatch
				+ "_g" + gamma + "_" + range + ".txt";
		
		ResultStat finalStat = new ResultStat();
		System.out.println("Start testing with " + numRepeat + " queries, range = " + range);
		for (int i = 0; i != numRepeat; ++i)
		{			
			ResultStat stat = bca.batchBCA_MoreEdge(queryArray[i],
						k, alpha, beta, gamma, threshold, range, queryBatch, docBatch, resultPath,
						sDocDocMatrix);			
			finalStat.addStat(stat);
			System.out.println(i + ": " + stat.get(0));
		}
		finalStat.avg();
		System.out.println("##Average Cost = " + finalStat.get(0));
		writer.println("##TextScore\t" + finalStat.get(2));	//"##TextScore\t"
		writer.println("##SimToTruth\t" + finalStat.get(3));
		writer.println("##MisMatch\t" + finalStat.get(4));
		writer.println("##DistScore\t" + finalStat.get(5));	//"##DistScore\t"
		writer.println("##Cost\t" + finalStat.get(0));	//"##Cost\t"
		writer.println("##Iter\t" + finalStat.get(1));
		writer.flush();
	}
	
	public void testClusterBCA (int NN, int MM, String clusterMethod, BCAWalker bca, Query [] randQueryArray,
			PrintWriter writer, String truthPathPrefix) throws IOException
	{
		int numCluster = NN * NN * MM;
		writer.println("***************************");
		writer.println("alpha = " + alpha + " epsilon = " + threshold);
		writer.println("beta = " + beta + " gamma = " + gamma + " range = " + range);
		writer.println(clusterMethod + ": " + NN + "\t" + MM + "\t" + numCluster);
		
		String truthPath = truthPathPrefix + "_Cluster.txt";
		String resultPath = this.resultPathPrefix + "test_" + clusterMethod + "_" + NN + "_" + MM
				+ "_g" + gamma + "_" + range + ".txt";
		
		Clusteror clusteror = new Clusteror(this.numQuery, this.numDoc, sQueryDocMatrix, sDocQueryMatrix, sRawQueryDocMatrix,
				this.docToLocArray, this.clusterPathPrefix);

		int [] docClusterArray = clusteror.getDocClusterArray(NN, MM, clusterMethod);
		int [] queryClusterArray = clusteror.getQueryClusterArray(NN, MM, docClusterArray, clusterMethod);

		Scanner truthScanner = null;
		PrintWriter truthWriter = null;
		boolean readTruth = new File(truthPath).exists() && (range < 1 || (NN > 1 || MM > 1) || threshold >= TRUTH_THRESHOLD);
		
		if (readTruth)
			truthScanner = new Scanner(new File(truthPath));
		else
			truthWriter = new PrintWriter(new File(truthPath));
		
		ResultStat finalStat = new ResultStat();
		System.out.println("Start testing with " + numRepeat + " queries, range = " + range);
		System.out.println("Alpha = " + alpha + " Beta = " + beta + " Epsilon = " + threshold);
		System.out.println("Method = " + clusterMethod + " N" + NN + " M" + MM);
		for (int i = 0; i != numRepeat; ++i)
		{
			Query query = randQueryArray[i];
			
			List<Integer> truthList = new ArrayList<Integer>();
			if (readTruth)
			{
				String truthLine = truthScanner.nextLine().trim();
				String [] truths = truthLine.split(" ");
				for (String truth : truths)
					truthList.add(Integer.parseInt(truth));
			}
			
			ResultStat stat = bca.clusterBCA(query,
					k, alpha, beta, gamma, threshold, range,
					docClusterArray, queryClusterArray, numCluster, numCluster, resultPath, truthList);
			
			
			if (!readTruth)
			{
				for (Integer truth : truthList)
					truthWriter.print(truth + " ");
				truthWriter.println();
				truthWriter.flush();
			}
			
			if (i > 0)	//TODO
				finalStat.addStat(stat);
			System.out.println(i + ": " + stat.get(0));

//			if (readTruth)
//			{
//				stat = bca.clusterBCA_Old(query,
//						k, alpha, beta, gamma, threshold, range,
//						docClusterArray, queryClusterArray, numCluster, numCluster, resultPath, truthList);
//				System.out.println("Old: " + stat.get(0));
//			}
			
			System.gc();
		}
		finalStat.avg();
		System.out.println("##Average Cost = " + finalStat.get(0));
		writer.println("##Cost\t" + finalStat.get(0));
		writer.println("##ReWeightCost\t" + (VectorWeighter.weightCost / numRepeat));
		writer.println("##NumReWeight\t" + (VectorWeighter.weightCount / numRepeat));
		VectorWeighter.weightCost = 0;
		VectorWeighter.weightCount = 0;
		writer.println("##rInk\t" + finalStat.get(6));
		writer.println("##TextScore\t" + finalStat.get(2));
		writer.println("##SimToTruth\t" + finalStat.get(3));
		writer.println("##MisMatch\t" + finalStat.get(4));
		writer.println("##CosSim\t" + finalStat.get(5));
		writer.println("##Iter\t" + finalStat.get(1));
		writer.println("##0.02\t" + finalStat.get(7));
		writer.println("##0.04\t" + finalStat.get(8));
		writer.println("##0.06\t" + finalStat.get(9));
		writer.println("##0.08\t" + finalStat.get(10));
		writer.println("##0.10\t" + finalStat.get(11));
		writer.println("##ExpectedQuery\t" + finalStat.get(12));
		writer.println("##ExpectedDoc\t" + finalStat.get(13));
		writer.flush();

		if (readTruth)
			truthScanner.close();
		else
			truthWriter.close();
		
		System.gc();
	}
	
	private void generateRandQuery(String testFilePath, int numRepeat, Query [] randQueryArray) throws IOException
	{
		File testFile = new File(testFilePath);
		if (testFile.isFile())
		{
			Scanner in = new Scanner(testFile);
			for (int i = 0; i < numRepeat; ++i)
			{
				Query query = new Query();
				int queryId = in.nextInt();
				query.queryId = queryId;
				
				if (this.AOL)
				{
					query.expectedQueryId = in.nextInt();
					query.expectedDocId = in.nextInt();
				}
				
				double locx = in.nextDouble();
				double locy = in.nextDouble();
				query.queryLoc = new Location(locx, locy);
				
				randQueryArray[i] = query;
			}
			in.close();
		}
		else
		{
			PrintWriter writer = new PrintWriter(new FileWriter(testFilePath), true);
			for (int i = 0; i < numRepeat; ++i)
			{
				randQueryArray[i] = new Query();
				
				List<Integer> clusterList = null;
				Session session = null;
				while (true)
				{
					if (AOL)
					{
						while (true)
						{
							int randQueryId = (int) (Math.random() * this.numQuery);
							List<Integer> sessionList = this.querySessionList.get(randQueryId);
							int sessionId = sessionList.get((int) (Math.random() * sessionList.size()));
							
							session = this.sessionList.get(sessionId);
							if (session.lastQueryId == randQueryId)
								continue;
							
							int expQueryId = session.lastQueryId;
							int expDocId = session.lastDocId;
							Location sessionLoc = this.docToLocArray[session.lastDocId];

							randQueryArray[i].queryId = randQueryId;
							randQueryArray[i].expectedQueryId = expQueryId;
							randQueryArray[i].expectedDocId = expDocId;
							randQueryArray[i].queryLoc = sessionLoc;
							break;
						}
					}
					else
					{
						while(true)
						{
							randQueryArray[i].queryId = (int) (Math.random() * this.numQuery);
							if (this.queryIdToStringArray[randQueryArray[i].queryId].split(" ").length == 1)
								break;
						}
					}
					
					//randQueryIdArray[i] = (int) (Math.random() * this.numQuery);

					clusterList = this.queryStringToClusterMap.get(this.queryIdToStringArray[randQueryArray[i].queryId]);
					if (clusterList.size() >= 5)
						break;
				}
				//int cellId = clusterList.get((int)(Math.random() * clusterList.size()));
				//randQueryLocArray[i] = Tools.cellIdToLoc(cellId, 10, 0.1);
				if (this.AOL == false)
				{
					System.out.println("Non-AOL: Random Location");
					randQueryArray[i].queryLoc = new Location(Math.random(), Math.random());
				}
				
				writer.print(randQueryArray[i].queryId + "\t");
				if (this.AOL == true)
				{
					writer.print(randQueryArray[i].expectedQueryId + "\t");
					writer.print(randQueryArray[i].expectedDocId + "\t");
				}
				writer.print(randQueryArray[i].queryLoc.getLocx() + "\t");
				writer.print(randQueryArray[i].queryLoc.getLocy());
			}
			writer.close();
		}
	}
	
	void DEXA (int k, Query [] randQueryArray, double r) throws IOException
	{
		//k = 1;
		
		String resultPath = this.resultPathPrefix + "DEXA.txt";
		PrintWriter writer = new PrintWriter(new FileWriter(resultPath, true));
		writer.println("r = " + r);
		
		Queue<Pair<Integer, Double>> resultHeap = new PriorityQueue<Pair<Integer, Double>>(k,
				new Comparator<Pair<Integer, Double>>() {
					@Override
					public int compare(Pair<Integer, Double> a, Pair<Integer, Double> b) {
						if (a.right > b.right)
							return 1;
						else if (a.right < b.right)
							return -1;
						else
							return 0;
					}
		});
		
        double tfidfScoreGlobal = 0;
        double cosScoreGlobal = 0;

		double [] validRange = {0.02, 0.04, 0.06, 0.08, 0.1};
		double [] numValidGlobal = new double[validRange.length];

		PrintWriter phraseWriter = new PrintWriter(new File(this.resultPathPrefix + "phrase_dexa" + r + ".txt"));
		for (Query query : randQueryArray)
		{
			int inputId = query.queryId;
			Location queryLoc = query.queryLoc;
			
			SVector inputVec = this.sQueryDocMatrix.getRowVector(inputId);
			Set<Integer> docSet = new HashSet<Integer>();
			for (Entry<Integer, Double> docEntry : inputVec)
				docSet.add(docEntry.getKey());
			
			for (int qId = 0; qId < this.numQuery; ++qId)
			{
				if (qId == inputId)
					continue;
				
				double score = 0;
				Set<Integer> docUnion = new HashSet<Integer>(docSet);
				
				SVector queryVec = this.sQueryDocMatrix.getRowVector(qId);
				for (Entry<Integer, Double> docEntry : queryVec)
				{
					int docId = docEntry.getKey();
					docUnion.add(docEntry.getKey());
					if (inputVec.contains(docId))
					{
						double dist = Tools.distance(queryLoc, this.docToLocArray[docId]) / Tools.SQRT2;
						score += Math.pow(2, -dist / r);
					}
				}
				score /= docUnion.size();
				
				resultHeap.add(new Pair<Integer, Double>(qId, score));
				if (resultHeap.size() > k)
					resultHeap.poll();
			}
			
			final int RESULT_SIZE = 10;
			final double RANGE = 0.1;
			Queue<Pair<Integer, Double>> docQueue = new PriorityQueue<Pair<Integer, Double>>(RESULT_SIZE, 
					new Comparator<Pair<Integer, Double>>() {
						@Override
						public int compare(Pair<Integer, Double> a, Pair<Integer, Double> b) {
							if (a.right > b.right)
								return 1;
							else if (a.right < b.right)
								return -1;
							else
								return 0;
						}
			});

			double [] numValid = new double[validRange.length];

			String inputString = this.queryIdToStringArray[inputId];
			String [] inputWords = inputString.split(" ");
			//retrieve top-$RESULT_SIZE documents w.r.t. the original input
			for (Entry<Integer, Double> entry : this.sQueryDocMatrix.getRowVector(inputId))
			{
				int docId = entry.getKey();
				
				double text = 0;
				if (AOL)
				{
					text = this.sRawQueryDocMatrix.get(inputId, docId);
				}
				else
				{
					for (String word : inputWords)
					{
						if (this.queryStringToIdMap.containsKey(word))
						{
							int id = this.queryStringToIdMap.get(word);
							text += this.sRawQueryDocMatrix.get(id, docId);					
						}
					}	
				}
								
				docQueue.add(new Pair<Integer, Double>(docId, text));
				if (docQueue.size() > RESULT_SIZE)
					docQueue.poll();
			}

			List<Double> originalScore = new ArrayList<Double>();
			List<Integer> originalResult = new ArrayList<Integer>();
			Set<String> originalWord = new HashSet<String>();
			while (docQueue.isEmpty() == false)
			{
				Pair<Integer, Double> doc = docQueue.poll();
				int docId = doc.left;
				double score = doc.right;

				originalScore.add(score);

				originalResult.add(docId);
				
				if (AOL == false)
				{
					String text = this.docToTextArray[docId];
					for (String word : text.split(" "))
						originalWord.add(word);
				}
			}
			
	        double tfidfScoreStat = 0;
	        double cosScoreStat = 0;
	        
			//writer.println(inputId + ": " + this.queryIdToStringArray[inputId] + " (" + queryLoc.getLocx() + ", " + queryLoc.getLocy() + ")");
			
	        double normSuggests = 1;
	        for (int i = 2; i <= k; ++i)
	        	normSuggests += 1.0 / Math.log(i);
			
			List<Pair<Integer, Double>> resultList = new ArrayList<Pair<Integer,Double>>();
			while (!resultHeap.isEmpty())
			{
				Pair<Integer, Double> topPair = resultHeap.poll();
				resultList.add(0, topPair);
			}
			
			phraseWriter.println(this.queryIdToStringArray[inputId]);
			phraseWriter.println();
			for (int i = 0; i < resultList.size(); ++i)
			{
				int phraseId = resultList.get(i).left;
				String phrase = this.queryIdToStringArray[phraseId];
				phraseWriter.println(phrase);
			}
			phraseWriter.println();
			
			for (int i = 0; i < resultList.size(); ++i)
			{
				int phraseId = resultList.get(i).left;
				//double score = resultList.get(i).right;
				
				String phrase = this.queryIdToStringArray[phraseId];
				String [] suggestWords = phrase.split(" ");

				SVector docVector = sQueryDocMatrix.getRowVector(phraseId);
				for (Entry<Integer, Double> docEntry : docVector)
				{
					int docId = docEntry.getKey();

					double dist = Tools.distance(queryLoc, this.docToLocArray[docId]) / Tools.SQRT2;
					
					for (int jj = validRange.length - 1; jj >= 0; --jj)
					{
						//largest to smallest
						if (dist <= validRange[jj])
							numValid[jj]++;
						else
							break;
					}
					
					double text = 0;
					if (AOL)
						text = this.sRawQueryDocMatrix.get(phraseId, docId);
					else
					{
						for (String word : suggestWords)
						{
							if (this.queryStringToIdMap.containsKey(word))
							{
								int id = this.queryStringToIdMap.get(word);
								text += this.sRawQueryDocMatrix.get(id, docId);						
							}
						}						
					}
				
					docQueue.add(new Pair<Integer, Double>(docId, text));
					if (docQueue.size() > RESULT_SIZE)
						docQueue.poll();
				}

//				SVector orgVec = new SVector();
//				SVector newVec = new SVector();
				
				Set<Integer> newResult = new HashSet<Integer>();
				int rank = docQueue.size();
				double cosScore = 0;
				while (docQueue.isEmpty() == false)
				{
					Pair<Integer, Double> docPair = docQueue.poll();
					int docId = docPair.left;
					
					if (!AOL && rank <= originalResult.size())
					{
						Set<String> wordSet = new HashSet<String>();
						newResult.add(docId);
						String text = this.docToTextArray[docId];
						for (String word : text.split(" "))
							wordSet.add(word);
						wordSet.addAll(originalWord);
						
						Map<String, Integer> wordIdMap = new HashMap<String, Integer>();
						int ii = 0;
						for (String word : wordSet)
							wordIdMap.put(word, ii++);
						
						SVector orgVec = new SVector();
						SVector newVec = new SVector();
						
						String originalString = this.docToTextArray[originalResult.get(rank - 1)];					
						for (String word : originalString.split(" "))
							orgVec.add(wordIdMap.get(word), 1);

						String newString = this.docToTextArray[docId];
						for (String word : newString.split(" "))
							newVec.add(wordIdMap.get(word), 1);
						
						double cosSim = orgVec.cosineSimilarity(newVec);
						if (rank > 1)
							cosSim /= Math.log(rank);
						cosScore += cosSim;
					}
					
					rank--;
				}
				
				double normDoc = 1;
				for (int ii = 2; ii < docQueue.size(); ++ii)
					normDoc += 1.0 / Math.log(ii);

				//cosScore = orgVec.cosineSimilarity(newVec);
				cosScore /= normDoc;
				
				for (Entry<Integer, Double> docEntry : docVector)
				{
					int docId = docEntry.getKey();
					double dist = Tools.distance(queryLoc, this.docToLocArray[docId]) / Tools.SQRT2;
					
					if (dist > RANGE)
						continue;					
					double text = 0;
					if (AOL)
						text = this.sRawQueryDocMatrix.get(phraseId, docId);
					else
					{
						for (String word : suggestWords)
						{
							if (this.queryStringToIdMap.containsKey(word))
							{
								int id = this.queryStringToIdMap.get(word);
								text += this.sRawQueryDocMatrix.get(id, docId);						
							}
						}
					}
				
					docQueue.add(new Pair<Integer, Double>(docId, text));
					if (docQueue.size() > RESULT_SIZE)
						docQueue.poll();
				}
				
				normDoc = 1;
				for (int ii = 2; ii < docQueue.size(); ++ii)
					normDoc += 1.0 / Math.log(ii);

				double tfidfScore = 0;
				rank = docQueue.size();
				while (docQueue.isEmpty() == false)
				{
					Pair<Integer, Double> doc = docQueue.poll();
					
					//double tfidfSuggest = this.sRawQueryDocMatrix.get(phraseId, doc.left);
					double tfidfSuggest = 0;
					for (String word : inputWords)
					{
						if (this.queryStringToIdMap.containsKey(word))
						{
							int id = this.queryStringToIdMap.get(word);
							tfidfSuggest += this.sRawQueryDocMatrix.get(id, doc.left);
						}
					}

					double ratio = tfidfSuggest;					
					if (rank <= originalScore.size())
						ratio /= originalScore.get(rank - 1);
					
					if (rank > 1)
						ratio /= Math.log(rank);
					tfidfScore += ratio;
					
					rank--;
				}
				
				tfidfScore /= normDoc;
				
				if (i > 0)
				{
					tfidfScore /= Math.log(i + 1);
					cosScore /= Math.log(i + 1);
				}
				
				tfidfScoreStat += tfidfScore;
				cosScoreStat += cosScore;
				
				//writer.println("[" + i + "] " + phrase + "\t" + score + "\t" + tfidfScore + "\t" + cosScore);
			}
			
			tfidfScoreStat /= normSuggests;
			cosScoreStat /= normSuggests;
			
	        tfidfScoreGlobal += tfidfScoreStat;
	        cosScoreGlobal += cosScoreStat;
	        
	        for (int i = 0; i < validRange.length; ++i)
	        {
	        	//writer.print((numValid[i] / k) + " ");
	     		numValidGlobal[i] += (numValid[i] / k);
	        }
	        //writer.println();
		}
		phraseWriter.close();
		
		tfidfScoreGlobal /= numRepeat;
		cosScoreGlobal /= numRepeat;
		
		writer.println("##TextScore\t" + tfidfScoreGlobal);
		writer.println("##CosSim\t" + cosScoreGlobal);
        for (int i = 0; i < validRange.length; ++i)
        	writer.println((numValidGlobal[i] / numRepeat));
		
		writer.close();
	}
}
