package sugg.cluster;

import sugg.matrix.SVector;
import sugg.tools.Timer;

public abstract class Cluster implements Comparable<Cluster> {
	protected SVector inkVector;
	protected SVector boundVector;
	public SVector tempVector;
	
	public double rInk;
	public double aInk;
	public double upperBound;
	public double maxInk;
	
	public double distInk;
	
	public double distCost;
	public double accuCost;
	
	protected Timer timer;
	
	public double statInk;
	public int statNumDist;
	
	public Cluster()
	{
		this.inkVector = new SVector();
		this.boundVector = new SVector();
		this.tempVector = new SVector();
		this.distCost = 0;
		this.accuCost = 0;
		this.rInk = this.aInk = 0;
		this.timer = new Timer();
		
		this.statNumDist = 0;
		this.statInk = 0;
	}
	
	public void addInk (int id, double ink, double inkBound, double threshold)
	{
		this.rInk += inkBound;
		//this.statInk += inkBound;
		
		
		if (ink > maxInk)
			this.maxInk = ink;

		if (this.boundVector.get(id) + inkBound > threshold)
			this.aInk += inkBound;
		
		inkBound = this.boundVector.get(id) + inkBound;
		this.boundVector.set(id, inkBound);
		
		if (inkBound > this.upperBound)
			this.upperBound = inkBound;
		
		ink = this.inkVector.get(id) + ink;
		this.inkVector.set(id, ink);
	}
	
	protected void clear() {
		this.rInk = 0;
		this.aInk = 0;
		this.maxInk = 0;
		this.upperBound = 0;
	}

	@Override
	public int compareTo (Cluster other) {
		//sort in descending order
		//if (this.rInk < other.rInk)
		//if (this.aInk < other.aInk)
		if (this.upperBound < other.upperBound)
			return 1;
		else
			return -1;
	}
}
