package sugg.cluster;

import java.io.File;
import java.io.IOException;
import java.util.Map.Entry;

import sugg.matrix.SMatrix;
import sugg.matrix.SVector;

public class ClusterMatrixTool {
	private int numQuery;
	private int numDoc;
	private int numCluster;
	private int numCell;
	private String docClusterMatrixPath;
	private String docCellMatrixPath;
	private String queryCellMatrixPath;
	
	public ClusterMatrixTool(int numQuery, int numDoc, int numCluster, int numCell, int NN, int MM, String clusterMatrixPath, String clusterMethod)
	{
		this.numQuery = numQuery;
		this.numDoc = numDoc;
		this.numCluster = numCluster;
		this.numCell = numCell;
		
		String clusterPrefix = clusterMatrixPath + clusterMethod + "_" + numCluster + 
				"_N" + NN + "_M" + MM;
		
		this.docClusterMatrixPath = clusterPrefix + "_doc_cluster_T.matrix";
		this.docCellMatrixPath = clusterPrefix + "_doc_cell_T.matrix";
		this.queryCellMatrixPath = clusterPrefix + "_query_cell_T.matrix";
	}
	
	public SMatrix sGetDocClusterMatrix(SMatrix sDocQueryMatrix, int [] queryToClusterArray)
					throws IOException
	{
		SMatrix sDocClusterMatrix;
		
		File docClusterMatrixFile = new File(docClusterMatrixPath);
		
		if (docClusterMatrixFile.isFile())
		{
			System.out.println("Reading Doc-Cluster-Matrix in " + docClusterMatrixPath);
			sDocClusterMatrix = new SMatrix(docClusterMatrixFile, true);
		}
		else
		{
			System.out.println("Building the Doc-Cluster-Matrix ... ");
			sDocClusterMatrix = new SMatrix(numDoc, numCluster);
			for (int dId = 0; dId != numDoc; ++dId)
			{
				SVector docQueryVector = sDocQueryMatrix.getRowVector(dId);
				for (Entry<Integer, Double> queryEntry : docQueryVector)
				{
					int qId = queryEntry.getKey();
					double weight = queryEntry.getValue();
					int clusterId = queryToClusterArray[qId];
					sDocClusterMatrix.set(dId, clusterId, sDocClusterMatrix.get(dId, clusterId) + weight);
				}
			}
			
			for (int i = 0; i != numDoc; ++i)
				sDocClusterMatrix.normalizeRow(i);
			
			//sDocClusterMatrix.write(docClusterMatrixFile, true);
		}
		
		return sDocClusterMatrix;
	}
	
	public SMatrix sGetDocCellMatrix(SMatrix sDocDocMatrix, int [] docToCellArray) throws IOException
	{
		if (sDocDocMatrix == null)
			return null;
		
		SMatrix sDocCellMatrix;
		
		File docCellMatrixFile = new File(docCellMatrixPath);
		
		if (docCellMatrixFile.isFile())
		{
			System.out.println("Reading Doc-Cell-Matrix in " + docCellMatrixPath);
			sDocCellMatrix = new SMatrix(docCellMatrixFile, true);
		}
		else
		{
			System.out.println("Building the DocCellMatrix ... ");
			sDocCellMatrix = new SMatrix(numDoc, numCell);
			
			for (int dId = 0; dId != numDoc; ++dId)
			{
				SVector docDocVector = sDocDocMatrix.getRowVector(dId);
				for (Entry<Integer, Double> docEntry : docDocVector)
				{
					int docId = docEntry.getKey();
					int cellId = docToCellArray[docId];
					double val = docEntry.getValue();
					
					sDocCellMatrix.set(dId, cellId, val + sDocCellMatrix.get(dId, cellId));
				}
			}
			
			for (int i = 0; i != numDoc; ++i)
				sDocCellMatrix.normalizeRow(i);
			
			//sDocCellMatrix.write(docCellMatrixFile, true);
		}
		
		return sDocCellMatrix;
	}
	
	public SMatrix sGetQueryCellMatrix(SMatrix sQueryDocMatrix, int [] docToCellArray) throws IOException
	{
		SMatrix queryCellMatrix;
		
		File queryCellMatrixFile = new File(queryCellMatrixPath);
		
		if (queryCellMatrixFile.isFile())
		{
			System.out.println("Reading Query-Cell-Matrix in " + queryCellMatrixPath);
		    queryCellMatrix = new SMatrix(queryCellMatrixFile, false);
		}
		else
		{
			System.out.println("Building the QueryCellMatrix ... ");
			queryCellMatrix = new SMatrix(numQuery, numCell);
			
			for (int qId = 0; qId != numQuery; ++qId)
			{
				SVector queryDocVecotor = sQueryDocMatrix.getRowVector(qId);
				for (Entry<Integer, Double> docEntry : queryDocVecotor)
				{
					int dId = docEntry.getKey();
					double val = docEntry.getValue();
					int cellId = docToCellArray[dId];
					queryCellMatrix.set(qId, cellId, val + queryCellMatrix.get(qId, cellId));
				}				
			}
			
			for (int i = 0; i != numQuery; ++i)
				queryCellMatrix.normalizeRow(i);
			
			//queryCellMatrix.write(queryCellMatrixFile, false);
		}
		
		return queryCellMatrix;
	}
}
