package sugg.cluster;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import sugg.matrix.SMatrix;
import sugg.matrix.SVector;
import sugg.tools.NodeWeighter;
import sugg.tools.SortedMap;
import sugg.tools.SortedMapTreeSet;
import sugg.tools.Tools;
import sugg.tools.VectorWeighter;

public class DocCluster extends Cluster
{
	public int cellId;
	
	public DocCluster(int id)
	{
		super();
		this.cellId = id;
	}
	
	private void sDistToDoc (
			SMatrix sQueryDocMatrix,
			NodeWeighter weighter,
			//int numDoc, double [] docWeight, double [] docDist, 
			double range,
			Map<Integer, ClusterVector> queryCellVectorMap,
			Map<Integer, SVector> queryDocVectorMap, 
			int [] docToCellArray, double threshold, double alpha, double beta)
	{	
		this.timer.start();
		Iterator<Entry<Integer, Double>> inkIter = this.inkVector.iterator();
		while(inkIter.hasNext())
		{
			Entry<Integer, Double> queryInkEntry = inkIter.next();
			int qId = queryInkEntry.getKey();
			double ink = queryInkEntry.getValue();

			if (this.boundVector.get(qId) <= threshold)
			//if (ink <= threshold)
				continue;
			
			SVector weightedQueryDocVector = VectorWeighter.sGetQueryDocVector(qId, 
					sQueryDocMatrix, weighter, beta, range,
					queryDocVectorMap);
//			SVector weightedQueryDocVector = WeightedVectorTools.sGetQueryDocVector(qId, 
//					sQueryDocMatrix, numDoc, docWeight, beta, docDist, range,
//					queryDocVectorMap);
			
			ClusterVector queryCellVector = queryCellVectorMap.get(qId);
			if (queryCellVector == null)
			{
				queryCellVector = new ClusterVector(weightedQueryDocVector, docToCellArray);
				queryCellVectorMap.put(qId, queryCellVector);
			}
			
			double distInk = 0;
			SVector cellVector = queryCellVector.getClusterVector(this.cellId);
			for (Entry<Integer, Double> docEntry : cellVector)
			{
				int docId = docEntry.getKey();
				double weight = docEntry.getValue();
				
				double newInk = (1 - alpha) * ink * weight;
				distInk += newInk;

				this.tempVector.add(docId, newInk);
			}
			this.statInk += distInk;

			assert(Tools.equal(distInk, this.boundVector.get(qId)));
			
//			SVector vector = null;
//			if (this.clusterMatrix == null)
//				vector = weightedQueryDocVector;
//			else
//				vector = this.clusterMatrix.getRowVector(qId);
//			
//			for (Entry<Integer, Double> docEntry : vector)
//			{
//				int docId = docEntry.getKey();
//				if (docId >= numDoc)
//					continue;
//				
//				if (docToCellArray[docId] != this.cellId)
//					continue;
//				
//				assert (docToCellArray[docId] == this.cellId);
//				
//				double weight = weightedQueryDocVector.get(docId);
//				
//				double newInk = (1 - alpha) * ink * weight;
//				this.statInk += newInk;
//
//				double qDj = this.tempVector.get(docId) + newInk;
////				if ((1 - alpha) * ink * weight >= this.upperBound + Tools.EP)
////					Tools.printError((1 - alpha) * ink * weight + "\t" + (this.upperBound + Tools.EP));
//				
//				//assert((1 - alpha) * ink * weight < this.upperBound + Tools.EP);
//				this.tempVector.set(docId, qDj);
//			}
			
			//queryInkEntry.setValue(0.0);
			inkIter.remove();
			this.boundVector.set(qId, 0.0);
		}
		this.distCost += this.timer.stop();
	}

	public boolean distribute(double alpha, double beta, double threshold, double range, 
			int numQuery, int numDoc, SortedMap<Integer, QueryCluster> sortedCluster,
			SMatrix sQueryDocMatrix, SMatrix sDocQueryMatrix, 
			Map<Integer, ClusterVector> queryCellVectorMap,
			Map<Integer, ClusterVector> docClusterVectorMap,
			Map<Integer, SVector> queryDocVectorMap, Map<Integer, SVector> docQueryVectorMap,
			int[] queryClusterArray, int[] docCellArray, 
			//double[] queryWeight, double[] docWeight, double[] queryDist, double[] docDist
			NodeWeighter weighter,
			SVector vistedDocVector) 
	{
		if (this.upperBound <= threshold)
			return false;

		boolean alive = false;		
		
		this.statNumDist += 1;
		
		this.sDistToDoc(sQueryDocMatrix, weighter, //numDoc, docWeight, docDist, 
				range, queryCellVectorMap, queryDocVectorMap, docCellArray, threshold, alpha, beta);
		
		this.timer.start();
		Iterator<Entry<Integer, Double>> qDIter = this.tempVector.iterator();
		while(qDIter.hasNext())
		{
			Entry<Integer, Double> qDEntry = qDIter.next();
			int dId = qDEntry.getKey();
			
			assert(docCellArray[dId] == this.cellId);
			
			double ink = qDEntry.getValue();
			
			if (ink <= threshold)
				continue;
			
			if (vistedDocVector != null)
				vistedDocVector.add(dId, 1);
			
//			SVector docQueryVector = WeightedVectorTools.sGetDocQueryVector(dId, 
//					sDocQueryMatrix, numQuery, queryWeight, beta, queryDist, range, docQueryVectorMap);
			SVector docQueryVector = VectorWeighter.sGetDocQueryVector(dId, 
					sDocQueryMatrix, weighter, beta, range, docQueryVectorMap);
			
			ClusterVector docClusterVector = docClusterVectorMap.get(dId);
			if (docClusterVector == null)
			{
				docClusterVector = new ClusterVector(docQueryVector, queryClusterArray);
				docClusterVectorMap.put(dId, docClusterVector);
			}
			
			for (Integer clusterId : docClusterVector.keySet())
			{
				double weight = docClusterVector.getClusterWeight(clusterId);
				Cluster cluster = sortedCluster.get(clusterId);
				double inkBound = ink * weight;
				cluster.addInk(dId, ink, inkBound, threshold);
				if (cluster.inkVector.get(dId) > threshold)
					alive = true;
			}
			
//			double docQueryNorm = docQueryVector.get(numQuery);
//			
//			SVector sDocClusterVector = sDocClusterMatrix.getRowVector(dId);
//			for (Entry<Integer, Double> clusterEntry : sDocClusterVector)
//			{
//				int clusterId = clusterEntry.getKey();
//				double weightBound = clusterEntry.getValue() / docQueryNorm;
//				
//				assert(weightBound > 0 && docQueryNorm > 0);
//				
//				Cluster cluster = sortedCluster.get(clusterId);
//				
//				weightBound = WeightedVectorTools.getWeight(weightBound, clusterWeight[clusterId], beta);
//				
//				double inkBound = ink * weightBound;
//				
//				cluster.addInk(dId, ink, inkBound, threshold);
//				
//				if (cluster.inkVector.get(dId) > threshold)
//					alive = true;
//			}
			
			//qDEntry.setValue(0.0);
			qDIter.remove();
		}
		this.accuCost += this.timer.stop();
		
		this.clear();
		
		return alive;
	}
	
	public boolean distribute(double alpha, double beta, double threshold, double range, 
			int numQuery, int numDoc, SortedMapTreeSet<Integer, QueryCluster> sortedCluster,
			SMatrix sQueryDocMatrix, SMatrix sDocQueryMatrix, 
			Map<Integer, ClusterVector> queryCellVectorMap,
			Map<Integer, ClusterVector> docClusterVectorMap,
			Map<Integer, SVector> queryDocVectorMap, Map<Integer, SVector> docQueryVectorMap,
			int[] queryClusterArray, int[] docCellArray, 
			//double[] queryWeight, double[] docWeight, double[] queryDist, double[] docDist
			NodeWeighter weighter,
			SVector vistedDocVector) 
	{
		if (this.upperBound <= threshold)
			return false;

		boolean alive = false;		
		
		this.statNumDist += 1;
		
		this.sDistToDoc(sQueryDocMatrix, weighter, //numDoc, docWeight, docDist, 
				range, queryCellVectorMap, queryDocVectorMap, docCellArray, threshold, alpha, beta);
		
		this.timer.start();
		Iterator<Entry<Integer, Double>> qDIter = this.tempVector.iterator();
		while(qDIter.hasNext())
		{
			Entry<Integer, Double> qDEntry = qDIter.next();
			int dId = qDEntry.getKey();
			
			assert(docCellArray[dId] == this.cellId);
			
			double ink = qDEntry.getValue();
			
			if (ink <= threshold)
				continue;
			
			if (vistedDocVector != null)
				vistedDocVector.add(dId, 1);
			
//			SVector docQueryVector = WeightedVectorTools.sGetDocQueryVector(dId, 
//					sDocQueryMatrix, numQuery, queryWeight, beta, queryDist, range, docQueryVectorMap);
			SVector docQueryVector = VectorWeighter.sGetDocQueryVector(dId, 
					sDocQueryMatrix, weighter, beta, range, docQueryVectorMap);
			
			ClusterVector docClusterVector = docClusterVectorMap.get(dId);
			if (docClusterVector == null)
			{
				docClusterVector = new ClusterVector(docQueryVector, queryClusterArray);
				docClusterVectorMap.put(dId, docClusterVector);
			}
			
			for (Integer clusterId : docClusterVector.keySet())
			{
				double weight = docClusterVector.getClusterWeight(clusterId);
				Cluster cluster = sortedCluster.get(clusterId);
				double inkBound = ink * weight;
				cluster.addInk(dId, ink, inkBound, threshold);
				if (cluster.inkVector.get(dId) > threshold)
					alive = true;
			}
			
//			double docQueryNorm = docQueryVector.get(numQuery);
//			
//			SVector sDocClusterVector = sDocClusterMatrix.getRowVector(dId);
//			for (Entry<Integer, Double> clusterEntry : sDocClusterVector)
//			{
//				int clusterId = clusterEntry.getKey();
//				double weightBound = clusterEntry.getValue() / docQueryNorm;
//				
//				assert(weightBound > 0 && docQueryNorm > 0);
//				
//				Cluster cluster = sortedCluster.get(clusterId);
//				
//				weightBound = WeightedVectorTools.getWeight(weightBound, clusterWeight[clusterId], beta);
//				
//				double inkBound = ink * weightBound;
//				
//				cluster.addInk(dId, ink, inkBound, threshold);
//				
//				if (cluster.inkVector.get(dId) > threshold)
//					alive = true;
//			}
			
			//qDEntry.setValue(0.0);
			qDIter.remove();
		}
		this.accuCost += this.timer.stop();
		
		this.clear();
		
		return alive;
	}
}
