package sugg.cluster;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import sugg.matrix.SMatrix;
import sugg.matrix.SVector;
import sugg.tools.NodeWeighter;
import sugg.tools.SortedMap;
import sugg.tools.SortedMapTreeSet;
import sugg.tools.Tools;
import sugg.tools.VectorWeighter;

public class QueryCluster extends Cluster {
	
	public int clusterId;
	
	public QueryCluster(int clusterId)
	{
		super();
		this.clusterId = clusterId;
	}

	public boolean distribute(int inputId, double alpha, double beta,
			double threshold, double range, int numQuery, int numDoc,
			SortedMap<Integer, Double> sortedResult,
			SortedMap<Integer, DocCluster> sortedCell,
			SMatrix sQueryDocMatrix,
			SMatrix sDocQueryMatrix,
			Map<Integer, ClusterVector> queryCellVectorMap,
			Map<Integer, ClusterVector> docClusterVectorMap,
			Map<Integer, SVector> queryDocVectorMap,
			Map<Integer, SVector> docQueryVectorMap,
			int[] queryClusterArray, int[] docCellArray,
			//double[] queryWeight, double[] docWeight, double[] queryDist, double[] docDist
			NodeWeighter weighter)
	{
		boolean alive = false;
		
		if (this.upperBound <= threshold)
			return false;
		
		this.statNumDist += 1;
		
		this.distInk = 0;

		this.timer.start();
		Iterator<Entry<Integer, Double>> docInkIter = this.inkVector.iterator();
		while(docInkIter.hasNext())
		{
			Entry<Integer, Double> docInkEntry = docInkIter.next();
			
			int dId = docInkEntry.getKey();
			double ink = docInkEntry.getValue();
			
			if (this.boundVector.get(dId) <= threshold)
			//if (ink <= threshold)
				continue;

//			SVector weightedDocQueryVector = WeightedVectorTools.sGetDocQueryVector(dId, 
//					sDocQueryMatrix, numQuery, queryWeight, beta, queryDist, range, docQueryVectorMap);

			SVector weightedDocQueryVector = VectorWeighter.sGetDocQueryVector(dId, 
					sDocQueryMatrix, weighter, beta, range, docQueryVectorMap);
			
			ClusterVector docClusterVector = docClusterVectorMap.get(dId);
			if (docClusterVector == null)
			{
				docClusterVector = new ClusterVector(weightedDocQueryVector, queryClusterArray);
				docClusterVectorMap.put(dId, docClusterVector);
			}
			
			double distInk = 0;
			SVector clusterVec = docClusterVector.getClusterVector(this.clusterId);
			for (Entry<Integer, Double> queryEntry : clusterVec)
			{
				int qId = queryEntry.getKey();
				double weight = queryEntry.getValue();
				
				double newInk = ink * weight;
				distInk += newInk;
				
				this.tempVector.add(qId, newInk);
			}
			
			this.statInk += distInk;
			
			assert(Tools.equal(distInk, this.boundVector.get(dId)));
			
//			SVector vector = null;
//			if (this.clusterMatrix != null)
//				vector = this.clusterMatrix.getRowVector(dId);
//			else
//				vector = weightedDocQueryVector;
//
//			for (Entry<Integer, Double> queryEntry : vector)
//			{
//				int queryId = queryEntry.getKey();
//				if (queryId >= numQuery)
//					continue;
//				
//				if (queryClusterArray[queryId] != this.clusterId)
//					continue;
//				
//				assert(queryClusterArray[queryId] == this.clusterId);
//					
//				double weight = weightedDocQueryVector.get(queryId);
//				
//				double newInk = ink * weight;
//				this.statInk += newInk;
//				
//				double qQi = this.tempVector.get(queryId) + ink * weight;
//
//				assert(ink * weight < this.upperBound + Tools.EP);
//	
//				this.tempVector.set(queryId, qQi);
//			}
			
			//docInkEntry.setValue(0.0);
			docInkIter.remove();
			this.boundVector.set(dId, 0.0);
		}
		this.distCost += this.timer.stop();
		
		this.timer.start();

		Iterator<Entry<Integer, Double>> tempQueryIter = this.tempVector.iterator();
		while(tempQueryIter.hasNext())
		{
			Entry<Integer, Double> tempQueryEntry = tempQueryIter.next();
			int qId = tempQueryEntry.getKey();
			
			assert(queryClusterArray[qId] == this.clusterId);
			
			double ink = tempQueryEntry.getValue();
			
			//System.out.println("qId = " + qId + " ink = " + ink);
			
			if (ink <= threshold)
				continue;
			
			//alive = true;
			

			if (sortedResult.get(qId) != null)
				sortedResult.put(qId, sortedResult.get(qId) + alpha * ink, true);
			else
				sortedResult.put(qId, alpha * ink, true);
			
			this.distInk += alpha * ink;
			
			SVector queryDocVector = VectorWeighter.sGetQueryDocVector(qId, 
					sQueryDocMatrix, weighter, beta, range,
					queryDocVectorMap);
//			SVector queryDocVector = WeightedVectorTools.sGetQueryDocVector(qId, 
//					sQueryDocMatrix, numDoc, docWeight, beta, docDist, range,
//					queryDocVectorMap);
			ClusterVector queryCellVector = queryCellVectorMap.get(qId);
			if (queryCellVector == null)
			{
				queryCellVector = new ClusterVector(queryDocVector, docCellArray);
				queryCellVectorMap.put(qId, queryCellVector);
			}
			
			for (Integer cellId : queryCellVector.keySet())
			{
				double weight = queryCellVector.getClusterWeight(cellId);
				double inkBound = (1 - alpha) * ink * weight;
				DocCluster cell = sortedCell.get(cellId);
				cell.addInk(qId, ink, inkBound, threshold);
				
				if (cell.inkVector.get(qId) > threshold)
					alive = true;
			}
			
//			double queryDocNorm = 0;
//			if (queryDocVector.numEntries() == 0)
//				continue;
//			else
//				queryDocNorm = queryDocVector.get(numDoc);
//
//			assert(queryDocNorm > 0);
//			
//			SVector sQueryCellVector = sQueryCellMatrix.getRowVector(qId);
//			for (Entry<Integer, Double> cellEntry : sQueryCellVector)
//			{
//				int cellId = cellEntry.getKey();
//				if (cellWeight[cellId] == 0)
//					continue;
//				
//				double weight = cellEntry.getValue() / queryDocNorm;
//				
//				Cluster cell = sortedCell.get(cellId);
//				
//				weight = WeightedVectorTools.getWeight(weight, cellWeight[cellId], beta);
//				
//				double inkBound = (1 - alpha) * ink * weight;
//
//				cell.addInk(qId, ink, inkBound, threshold);
//				
//				if (cell.inkVector.get(qId) > threshold)
//					alive = true;
//			}
			
			tempQueryIter.remove();
			//tempQueryEntry.setValue(0.0);
		}
		//this.qQTemp.clear();
		
		this.accuCost += timer.stop();

		this.clear();
		
		return alive;

	}
	
	public boolean distribute(int inputId, double alpha, double beta,
			double threshold, double range, int numQuery, int numDoc,
			SortedMapTreeSet<Integer, Double> sortedResult,
			SortedMapTreeSet<Integer, DocCluster> sortedCell,
			SMatrix sQueryDocMatrix,
			SMatrix sDocQueryMatrix,
			Map<Integer, ClusterVector> queryCellVectorMap,
			Map<Integer, ClusterVector> docClusterVectorMap,
			Map<Integer, SVector> queryDocVectorMap,
			Map<Integer, SVector> docQueryVectorMap,
			int[] queryClusterArray, int[] docCellArray,
			//double[] queryWeight, double[] docWeight, double[] queryDist, double[] docDist
			NodeWeighter weighter)
	{
		boolean alive = false;
		
		if (this.upperBound <= threshold)
			return false;
		
		this.statNumDist += 1;
		
		this.distInk = 0;

		this.timer.start();
		Iterator<Entry<Integer, Double>> docInkIter = this.inkVector.iterator();
		while(docInkIter.hasNext())
		{
			Entry<Integer, Double> docInkEntry = docInkIter.next();
			
			int dId = docInkEntry.getKey();
			double ink = docInkEntry.getValue();
			
			if (this.boundVector.get(dId) <= threshold)
			//if (ink <= threshold)
				continue;

//			SVector weightedDocQueryVector = WeightedVectorTools.sGetDocQueryVector(dId, 
//					sDocQueryMatrix, numQuery, queryWeight, beta, queryDist, range, docQueryVectorMap);

			SVector weightedDocQueryVector = VectorWeighter.sGetDocQueryVector(dId, 
					sDocQueryMatrix, weighter, beta, range, docQueryVectorMap);
			
			ClusterVector docClusterVector = docClusterVectorMap.get(dId);
			if (docClusterVector == null)
			{
				docClusterVector = new ClusterVector(weightedDocQueryVector, queryClusterArray);
				docClusterVectorMap.put(dId, docClusterVector);
			}
			
			double distInk = 0;
			SVector clusterVec = docClusterVector.getClusterVector(this.clusterId);
			for (Entry<Integer, Double> queryEntry : clusterVec)
			{
				int qId = queryEntry.getKey();
				double weight = queryEntry.getValue();
				
				double newInk = ink * weight;
				distInk += newInk;
				
				this.tempVector.add(qId, newInk);
			}
			
			this.statInk += distInk;
			
			assert(Tools.equal(distInk, this.boundVector.get(dId)));
			
//			SVector vector = null;
//			if (this.clusterMatrix != null)
//				vector = this.clusterMatrix.getRowVector(dId);
//			else
//				vector = weightedDocQueryVector;
//
//			for (Entry<Integer, Double> queryEntry : vector)
//			{
//				int queryId = queryEntry.getKey();
//				if (queryId >= numQuery)
//					continue;
//				
//				if (queryClusterArray[queryId] != this.clusterId)
//					continue;
//				
//				assert(queryClusterArray[queryId] == this.clusterId);
//					
//				double weight = weightedDocQueryVector.get(queryId);
//				
//				double newInk = ink * weight;
//				this.statInk += newInk;
//				
//				double qQi = this.tempVector.get(queryId) + ink * weight;
//
//				assert(ink * weight < this.upperBound + Tools.EP);
//	
//				this.tempVector.set(queryId, qQi);
//			}
			
			//docInkEntry.setValue(0.0);
			docInkIter.remove();
			this.boundVector.set(dId, 0.0);
		}
		this.distCost += this.timer.stop();
		
		this.timer.start();

		Iterator<Entry<Integer, Double>> tempQueryIter = this.tempVector.iterator();
		while(tempQueryIter.hasNext())
		{
			Entry<Integer, Double> tempQueryEntry = tempQueryIter.next();
			int qId = tempQueryEntry.getKey();
			
			assert(queryClusterArray[qId] == this.clusterId);
			
			double ink = tempQueryEntry.getValue();
			
			//System.out.println("qId = " + qId + " ink = " + ink);
			
			if (ink <= threshold)
				continue;
			
			//alive = true;
			

			if (sortedResult.get(qId) != null)
				sortedResult.put(qId, sortedResult.get(qId) + alpha * ink);
			else
				sortedResult.put(qId, alpha * ink);
			
			this.distInk += alpha * ink;
			
			SVector queryDocVector = VectorWeighter.sGetQueryDocVector(qId, 
					sQueryDocMatrix, weighter, beta, range,
					queryDocVectorMap);
//			SVector queryDocVector = WeightedVectorTools.sGetQueryDocVector(qId, 
//					sQueryDocMatrix, numDoc, docWeight, beta, docDist, range,
//					queryDocVectorMap);
			ClusterVector queryCellVector = queryCellVectorMap.get(qId);
			if (queryCellVector == null)
			{
				queryCellVector = new ClusterVector(queryDocVector, docCellArray);
				queryCellVectorMap.put(qId, queryCellVector);
			}
			
			for (Integer cellId : queryCellVector.keySet())
			{
				double weight = queryCellVector.getClusterWeight(cellId);
				double inkBound = (1 - alpha) * ink * weight;
				DocCluster cell = sortedCell.get(cellId);
				cell.addInk(qId, ink, inkBound, threshold);
				
				if (cell.inkVector.get(qId) > threshold)
					alive = true;
			}
			
//			double queryDocNorm = 0;
//			if (queryDocVector.numEntries() == 0)
//				continue;
//			else
//				queryDocNorm = queryDocVector.get(numDoc);
//
//			assert(queryDocNorm > 0);
//			
//			SVector sQueryCellVector = sQueryCellMatrix.getRowVector(qId);
//			for (Entry<Integer, Double> cellEntry : sQueryCellVector)
//			{
//				int cellId = cellEntry.getKey();
//				if (cellWeight[cellId] == 0)
//					continue;
//				
//				double weight = cellEntry.getValue() / queryDocNorm;
//				
//				Cluster cell = sortedCell.get(cellId);
//				
//				weight = WeightedVectorTools.getWeight(weight, cellWeight[cellId], beta);
//				
//				double inkBound = (1 - alpha) * ink * weight;
//
//				cell.addInk(qId, ink, inkBound, threshold);
//				
//				if (cell.inkVector.get(qId) > threshold)
//					alive = true;
//			}
			
			tempQueryIter.remove();
			//tempQueryEntry.setValue(0.0);
		}
		//this.qQTemp.clear();
		
		this.accuCost += timer.stop();

		this.clear();
		
		return alive;

	}
}