package sugg.invertedindex;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Map;
import java.util.Set;

import org.apache.jdbm.DBMaker;

import sugg.tools.LRUBuffer;

public class InvertedIndexIsolated extends InvertedIndex
{
	private Map<String, Long> treeMap;
	private RandomAccessFile dataFile;
	
	public InvertedIndexIsolated (String treeFilePrefix, String treeName, boolean build) throws FileNotFoundException
	{
		super();
		
		String treeFilePath = treeFilePrefix + ".db";
		String dataFilePath = treeFilePrefix + ".data";
		
		if (build == true)
		{
			System.out.print("Building inverted index in " + treeFilePath);
			db = DBMaker.openFile(treeFilePath).disableTransactions().closeOnExit().make();
			this.treeMap = db.createTreeMap(treeName);
			this.dataFile = new RandomAccessFile(dataFilePath, "rw");
		}
		else
		{
			System.out.print("Reading inverted index in " + treeFilePath);
			this.db = DBMaker.openFile(treeFilePath).disableTransactions().closeOnExit().make();
			this.treeMap = db.getTreeMap(treeName);
			this.dataFile = new RandomAccessFile(dataFilePath, "r");
		}

		int bufferCapacity = (int) (this.size() * 0.05);
		this.buffer = new LRUBuffer<String, PhraseInfo>(bufferCapacity);
		System.out.println(" (buffer = " + bufferCapacity + ")");
	}

	@Override
	public Set<String> keySet ()
	{
		return this.treeMap.keySet();
	}
	
	@Override
	public PhraseInfo get (String phrase) throws IOException
	{
		this.timer.start();
		
		PhraseInfo phraseInfo = this.buffer.get(phrase);
		
		if (phraseInfo == null)
		{
			phraseInfo = new PhraseInfo();
			long pos = treeMap.get(phrase);
			dataFile.seek(pos);
			
			phraseInfo.id = dataFile.readLong();
			phraseInfo.idf = dataFile.readFloat();
			phraseInfo.tfSum = dataFile.readLong();
			int postSize = dataFile.readInt();
			for (int i = 0; i != postSize; ++i)
			{
				int docId = dataFile.readInt();
				int tf = dataFile.readInt();
				phraseInfo.postList.add(new Posting(docId, tf));
			}
			
			this.buffer.put(phrase, phraseInfo);
		}
		
		this.cost += timer.stop();
		
		return phraseInfo;
	}

	@Override
	public void put (String phrase, PhraseInfo phraseInfo) throws IOException
	{
		treeMap.put(phrase, dataFile.getFilePointer());
		
		dataFile.writeLong(phraseInfo.id);
		dataFile.writeFloat(phraseInfo.idf);
		dataFile.writeLong(phraseInfo.tfSum);
		dataFile.writeInt(phraseInfo.postList.size());
		for (Posting posting : phraseInfo.postList)
		{
			dataFile.writeInt(posting.getId());
			dataFile.writeInt(posting.getTf());
		}
	}

	@Override
	public int size()
	{
		return this.treeMap.size();
	}

	@Override
	public void close() throws IOException
	{
		this.db.close();
		this.dataFile.close();
	}
}
