package sugg.invertedindex;

import java.util.Map;
import java.util.Set;

import org.apache.jdbm.DBMaker;

import sugg.tools.LRUBuffer;

public class InvertedIndexPacked extends InvertedIndex
{
	private Map<String, PhraseInfo> treeMap;
	
	public InvertedIndexPacked (String treeFilePrefix, String treeName, boolean build)
	{
		super();
		
		String treeFilePath = treeFilePrefix + ".db";
		
		if (build == true)
		{
			System.out.print("Building inverted index in " + treeFilePath);
			db = DBMaker.openFile(treeFilePath).disableTransactions().closeOnExit().make();
			this.treeMap = db.createTreeMap(treeName);
		}
		else
		{
			System.out.print("Reading inverted index in " + treeFilePath);
			this.db = DBMaker.openFile(treeFilePath).disableTransactions().closeOnExit().make();
			this.treeMap = db.getTreeMap(treeName);
		}
	
		int bufferCapacity = (int) (0);
		this.buffer = new LRUBuffer<String, PhraseInfo>(bufferCapacity);
		System.out.println(" (buffer = " + bufferCapacity + ")");
	}

	@Override
	public Set<String> keySet ()
	{
		return this.treeMap.keySet();
	}
	
	@Override
	public PhraseInfo get (String phrase)
	{
		this.timer.start();
		
		PhraseInfo phraseInfo = this.buffer.get(phrase);
		
		if (phraseInfo == null)
		{
			phraseInfo = this.treeMap.get(phrase);
			this.buffer.put(phrase, phraseInfo);
		}
		
		this.cost += timer.stop();
		
		return phraseInfo;
	}

	@Override
	public void put (String phrase, PhraseInfo phraseInfo)
	{
		treeMap.put(phrase, phraseInfo);
	}
	
	@Override
	public int size()
	{
		return this.treeMap.size();
	}

	@Override
	public void close()
	{
		this.db.close();
	}
}
