package sugg.objects;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/*
 * Thanks to
 * http://stackoverflow.com/questions/714796/priorityqueue-heap-update
 * Modified by Shuyao
 * */

public class Heap<T> {

	private List<T> list;
	private Comparator<T> comparator;

	public Heap(Comparator<T> comparator) {
		this.list = new ArrayList<T>();
		this.comparator = comparator;
	}

	public void push(T obj) {
		list.add(obj);
		pushUp(list.size() - 1);
	}

	public T pop() {
		if (list.size() > 0) {
			swap(0, list.size() - 1);
			T result = list.remove(list.size() - 1);
			pushDown(0);
			return result;
		} else {
			return null;
		}
	}

	public T getFirst() {
		return list.get(0);
	}

	public T get(int index) {
		return list.get(index);
	}

	public int size() {
		return list.size();
	}

	private boolean isGreaterOrEqual(int first, int last) {
		return this.comparator.compare(this.list.get(first),
				this.list.get(last)) > 0;
	}

	protected int parent(int i) {
		return (i - 1) / 2;
	}

	protected int left(int i) {
		return 2 * i + 1;
	}

	protected int right(int i) {
		return 2 * i + 2;
	}

	protected void swap(int i, int j) {
		T tmp = list.get(i);
		list.set(i, list.get(j));
		list.set(j, tmp);
	}

	public void pushDown(int i) {
		int left = left(i);
		int right = right(i);
		int largest = i;

		if (left < list.size() && !isGreaterOrEqual(largest, left)) {
			largest = left;
		}
		if (right < list.size() && !isGreaterOrEqual(largest, right)) {
			largest = right;
		}

		if (largest != i) {
			swap(largest, i);
			pushDown(largest);
		}
	}

	public void pushUp(int i) {
		while (i > 0 && !isGreaterOrEqual(parent(i), i)) {
			swap(parent(i), i);
			i = parent(i);
		}
	}

	public String toString() {
		StringBuffer s = new StringBuffer("Heap:\n");
		int rowStart = 0;
		int rowSize = 1;
		for (int i = 0; i < list.size(); i++) {
			if (i == rowStart + rowSize) {
				s.append('\n');
				rowStart = i;
				rowSize *= 2;
			}
			s.append(get(i));
			s.append(" ");
		}
		return s.toString();
	}
}