package sugg.tools;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.PriorityQueue;
import java.util.Scanner;
import java.util.Set;

import sugg.DataGenerator.Cell;
import sugg.matrix.SMatrix;
import sugg.matrix.SVector;
import sugg.objects.Location;
import sugg.objects.Session;

public class AOL {
	String dataPathPrefix = "data/AOL/";
	String outputPathPrefix = "output/";

	public void parseData(String inputType) throws IOException
	{
		this.outputPathPrefix += inputType + "/";
		Tools.mkdir(this.outputPathPrefix);
		
		/** Skip if the matrices are already built **/
		String rawMatrixFile = this.outputPathPrefix + "matrix/query_doc.raw.matrix";
		if (new File(rawMatrixFile).exists())
			return;
		
		int numFile = 10;
		if (inputType.contains("_"))
		{
			numFile = Integer.parseInt(inputType.split("_")[1]);
			if (numFile < 1 || numFile > 10)
			{
				Tools.printError("NumFile OutOfRange [1 - 10]");
				System.exit(1);
			}
		}
		
		double minX = 29;
		double maxX = 49;
		double minY = -125;
		double maxY = -70;
		
		Map<String, Location> urlLocationMap = new HashMap<String, Location>();
		Scanner urlLocScanner = new Scanner(new File(dataPathPrefix + "urlloc.txt"));
		while (urlLocScanner.hasNextLine())
		{
			String url = urlLocScanner.nextLine();
			double locx = Double.parseDouble(urlLocScanner.nextLine());
			double locy = Double.parseDouble(urlLocScanner.nextLine());
			
			if (locx >= minX && locx <= maxX && locy >= minY && locy <= maxY)
			{
				locx = (locx - minX) / (maxX - minX);
				locy = (locy - minY) / (maxY - minY);
				urlLocationMap.put(url, new Location(locx, locy));
			}
		}
		urlLocScanner.close();
		
		Map<String, Integer> queryIdMap = new HashMap<String, Integer>();
		
		for (int i = 1; i <= numFile; ++i)
		{
			String file = "user-ct-test-collection-0" + i + ".txt";
			if (i >= 10)
				file = "user-ct-test-collection-" + i + ".txt";
			String filePath = this.dataPathPrefix + file;
			
			Scanner in = new Scanner(new File(filePath));
			in.nextLine();
			while (in.hasNextLine())
			{
				String line = in.nextLine().trim();
				String [] split = line.split("\t");
				if (split.length != 5)
					continue;
				
				String query = split[1];
				
				if (query.length() <= 3)
					continue;
				
				String url = split[4];
				if (urlLocationMap.containsKey(url) == false)
					continue;
				
				int count = 0;
				if (queryIdMap.containsKey(query))
					count = queryIdMap.get(query);
				
				queryIdMap.put(query, count + 1);
			}
			in.close();
		}

		Iterator<Entry<String, Integer>> iter = queryIdMap.entrySet().iterator();
		while (iter.hasNext())
		{
			Entry<String, Integer> queryEntry = iter.next();
			int count = queryEntry.getValue();
			if (count <= 3)
				iter.remove();
		}

		Map<String, Integer> docIdMap = new HashMap<String, Integer>();
		for (int i = 1; i <= numFile; ++i)
		{
			String file = "user-ct-test-collection-0" + i + ".txt";
			if (i >= 10)
				file = "user-ct-test-collection-" + i + ".txt";
			
			String filePath = this.dataPathPrefix + file;
			
			Scanner in = new Scanner(new File(filePath));
			in.nextLine();
			while (in.hasNextLine())
			{
				String line = in.nextLine().trim();
				String [] split = line.split("\t");
				if (split.length != 5)
					continue;
				
				String query = split[1];
				String url = split[4];
				
				if (queryIdMap.containsKey(query) == false)
					continue;
				if (urlLocationMap.containsKey(url) == false)
					continue;
				
				int count = -1;
				if (docIdMap.containsKey(url))
					count = docIdMap.get(url);
				docIdMap.put(url, count + 1);
			}
			in.close();
		}
		
		List<String> queryList = new ArrayList<String>();
		for (Entry<String, Integer> queryEntry : queryIdMap.entrySet())
		{
			String query = queryEntry.getKey();
			queryEntry.setValue(queryList.size());
			queryList.add(query);
		}
		
		List<String> docList = new ArrayList<String>();
		for (Entry<String, Integer> docEntry : docIdMap.entrySet())
		{
			String url = docEntry.getKey();
			docEntry.setValue(docList.size());
			docList.add(url);
		}

		int numQuery = queryList.size();
		int numDoc = docList.size();

		System.out.println("numQuery = " + numQuery);
		System.out.println("numDoc = " + numDoc);
		
		Scanner in = new Scanner(new File("data/location.txt"));
		List<Location> locList = new ArrayList<Location>();
		while(in.hasNextDouble())
		{
			double locx = in.nextDouble();
			double locy = in.nextDouble();
			locList.add(new Location(locx, locy));
		}
		in.close();
		
		SMatrix rawQueryDocMatrix = new SMatrix(numQuery, numDoc);

		Location [] docLocArray = new Location[numDoc];
		int sessionId = 0;
		int lastUserId = -1;
		int lastDocId = -1;
		int lastQueryId = -1;
		List<Session> sessionList = new ArrayList<Session>();
		List<Set<Integer>> querySessionList = new ArrayList<Set<Integer>>();
		for (int i = 0; i < numQuery; ++i)
			querySessionList.add(new HashSet<Integer>());
		for (int i = 1; i <= numFile; ++i)
		{
			String file = "user-ct-test-collection-0" + i + ".txt";
			if (i >= 10)
				file = "user-ct-test-collection-" + i + ".txt";
			
			String filePath = this.dataPathPrefix + file;

			in = new Scanner(new File(filePath));
			in.nextLine();
			while (in.hasNextLine())
			{
				String line = in.nextLine().trim();
				String [] split = line.split("\t");
				if (split.length != 5)
					continue;
				
				int userId = Integer.parseInt(split[0]);
				String query = split[1];
				String url = split[4];
				
				if (queryIdMap.containsKey(query) == false)
					continue;
				if (urlLocationMap.containsKey(url) == false)
					continue;
				
				int queryId = queryIdMap.get(query);
				int docId = docIdMap.get(url);
				
				if (lastUserId == -1)
					lastUserId = userId;
				
				if (userId != lastUserId)
				{
					assert(lastQueryId > 0 && lastDocId > 0);
					Session session = new Session(sessionId, lastQueryId, lastDocId);
					sessionList.add(session);
					
					sessionId++;
				}
				
				querySessionList.get(queryId).add(sessionId);
				
//				Location loc = locList.get((int) (Math.random() * locList.size()));
//				double locx = loc.getLocx() * (1 + 0.02 * Math.random() - 0.01);
//				double locy = loc.getLocy() * (1 + 0.02 * Math.random() - 0.01);
//				if (locx < 0)
//					locx = 0;
//				if (locy < 0)
//					locy = 0;
//				if (locx > 1)
//					locx = 1;
//				if (locy > 1)
//					locy = 1;
//				docLocArray[docId] = new Location(locx, locy);
				//docLocArray[docId] = new Location(Math.random(), Math.random());
				docLocArray[docId] = urlLocationMap.get(url);
				
				rawQueryDocMatrix.add(queryId, docId, 1);
				
				lastQueryId = queryId;
				lastDocId = docId;
				lastUserId = userId;
			}
			sessionList.add(new Session(sessionId, lastQueryId, lastDocId));
			
			in.close();
		}


		String docLocFilePath = this.outputPathPrefix + "docloc.txt";
		PrintWriter writer = new PrintWriter(new File(docLocFilePath));
		for (int i = 0; i != numDoc; ++i)
		{
			writer.println(docLocArray[i].getLocx() + "\t" + docLocArray[i].getLocy());
		}
		writer.close();
		
		String querySessionFilePath = this.outputPathPrefix + "querysession.txt";
		writer = new PrintWriter(new File(querySessionFilePath));
		for (Set<Integer> sessionSet : querySessionList)
		{
			writer.print(sessionSet.size());
			for (Integer sId : sessionSet)
				writer.print("\t" + sId);
			writer.println();
		}
		writer.close();
		
		String sessionFilePath = this.outputPathPrefix + "session.txt";
		writer = new PrintWriter(new File(sessionFilePath));
		writer.println(sessionList.size());
		for (Session session : sessionList)
			writer.println(session.sessionId + "\t" + session.lastQueryId + "\t" + session.lastDocId);
		writer.close();
		
		String queryFilePath = this.outputPathPrefix + "querystring.txt";
		writer = new PrintWriter(new File(queryFilePath));
		for (String query : queryList)
		{
			writer.println(query);
			int queryId = queryIdMap.get(query);
			List<Integer> clusterList = this.getClusterList(10, docLocArray, 
					rawQueryDocMatrix.getRowVector(queryId));
			for (int cellId : clusterList)
				writer.print(cellId + " ");
			writer.println();
		}
		writer.close();
		
		String docFilePath = this.outputPathPrefix + "doctext.txt";
		writer = new PrintWriter(new File(docFilePath));
		for (String url : docList)
			writer.println(url);
		writer.close();
		
		String infoFilePath = this.outputPathPrefix + "info.txt";
		writer = new PrintWriter(new File(infoFilePath));
		writer.println("numQuery = " + numQuery);
		writer.println("numEdge = " + rawQueryDocMatrix.numEntries());
		writer.println("numDoc = " + numDoc);
		writer.close();
		
		Tools.mkdir(this.outputPathPrefix + "matrix/");
//		String queryDocMatrixFile = this.outputPathPrefix + "matrix/query_doc.matrix";
//		String docQueryMatrixFile = this.outputPathPrefix + "matrix/doc_query.matrix";

		System.out.println("Writing RawQueryDocMatrix ...");
		rawQueryDocMatrix.write(new File(rawMatrixFile), false);
		
//		SMatrix queryDocMatrix = rawQueryDocMatrix;
//		SMatrix docQueryMatrix = rawQueryDocMatrix.transpose();
//		
//		for (int i = 0; i != numQuery; ++i)
//			queryDocMatrix.normalizeRow(i);
//		for (int i = 0; i != numDoc; ++i)
//			docQueryMatrix.normalizeRow(i);
//
//		System.out.println("Writing QueryDocMatrix ...");
//		queryDocMatrix.write(new File(queryDocMatrixFile), false);
//
//		System.out.println("Writing DocQueryMatrix ...");
//		docQueryMatrix.write(new File(docQueryMatrixFile), true);
	}
	
	List<Integer> getClusterList (int cellN, Location [] docLocArray, SVector queryDocVector)
	{
		Cell [] cellArray = new Cell[cellN * cellN];
		for (int i = 0; i != cellN * cellN; ++i)
			cellArray[i] = new Cell(i, cellN, 1);

		for (Entry<Integer, Double> docEntry : queryDocVector)
		{
			int docId = docEntry.getKey();
			int cellId = Tools.locToCellID(docLocArray[docId], cellN, 1.0/cellN);
			cellArray[cellId].docList.add(docId);
		}
		
		int c = 5;	//5 cluster cells
		PriorityQueue<Cell> clusetCellHeap = new PriorityQueue<Cell>();
		for (int i = 0; i != cellN * cellN; ++i)
		{
			clusetCellHeap.add(cellArray[i]);
			if (clusetCellHeap.size() > c)
				clusetCellHeap.poll();
		}
		
		List<Integer> clusterList = new ArrayList<Integer>();
		while (clusetCellHeap.isEmpty() == false)
		{
			Cell cell = clusetCellHeap.peek();
			clusetCellHeap.poll();
			if (cell.getWeight() == 0)
				continue;
			clusterList.add(cell.id);
		}
		
		return clusterList;
	}
}
