package sugg.tools;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class LRUBuffer<Key, Value>
{
	private Map<Key, Value> map;
	private List<Key> list;
	private int capacity;
	
	private int hit;
	private int miss;
	
	public LRUBuffer(int capacity)
	{
		this.capacity = capacity;
		this.map = new HashMap<Key, Value>(capacity);
		this.list = new LinkedList<Key>();
		this.hit = this.miss = 0;
	}
	
	public Value get (Key key)
	{
		if (this.capacity == 0)
			return null;
		
		if (this.map.containsKey(key))
		{
			this.hit++;
			this.list.remove(key);
			this.list.add(key);
			return this.map.get(key);
		}
		
		this.miss++;
		return null;
	}
	
	public void put (Key key, Value value)
	{
		if (this.capacity == 0)
			return;
		
		if (this.map.containsKey(key))
		{
			this.list.remove(key);
			this.list.add(key);
			return;
		}
		
		if (this.list.size() == this.capacity)
		{
			this.map.remove(this.list.get(0));
			this.list.remove(0);
		}
		this.list.add(key);
		this.map.put(key, value);
	}
	
	public int getHit()
	{
		return this.hit;
	}
	
	public int getMiss()
	{
		return this.miss;
	}

	public int size()
	{
		return this.list.size();
	}
	
	public int capacity()
	{
		return this.capacity;
	}
	
	public void print()
	{
		System.out.print("*DEBUG* ");
		for (Key key : this.list)
			System.out.print(" (" + key + " " + this.map.get(key) + ") ");
		System.out.println();
	}
}
