package sugg.tools;

import java.util.Map.Entry;

import sugg.matrix.SMatrix;
import sugg.objects.Location;

public class NodeWeighter {
	double [] docDist;
	double [] queryDist;
	
	double [] docWeight;
	double [] queryWeight;
	
	Location [] docLocArray;
	int numQuery;
	int numDoc;
	Location queryLoc;
	double gamma;
	SMatrix sQueryDocMatrix;
	
	double maxDist = 1;	//default: 1
	double maxWeight = 1;
	double minWeight = 0;
	
	public int docDistCalc = 0;
	public int queryDistCalc = 0;
	
	public NodeWeighter(int numQuery, int numDoc, Location queryLoc, Location [] docLocArray, double gamma, SMatrix sQueryDocMatrix)
	{
		this.docLocArray = docLocArray;
		this.numQuery = numQuery;
		this.numDoc = numDoc;
		this.queryLoc = queryLoc;
		this.gamma = gamma;
		this.sQueryDocMatrix = sQueryDocMatrix;
		
		this.docDist = new double[numDoc];
		this.docWeight = new double[numDoc];
		for (int i = 0; i < numDoc; ++i)
		{
			this.docDist[i] = -1;
			this.docWeight[i] = -1;

//			this.docDist[i] = this.getDocDist(i);
//			this.docWeight[i] = this.getDocWeight(i);
		}
//		Tools.normalizeArrayTo01(this.docWeight);
		
		this.queryDist = new double[numQuery];
		this.queryWeight = new double[numQuery];
		for (int i = 0; i < numQuery; ++i)
		{
			this.queryDist[i] = -1;
			this.queryWeight[i] = -1;
			
//			this.queryDist[i] = this.getQueryDist(i);
//			this.queryWeight[i] = this.getQueryWeight(i);
		}
//		Tools.normalizeArrayTo01(this.queryWeight);

//		double locx = queryLoc.getLocx();
//		double locy = queryLoc.getLocy();
//		if (locx <= 0.5 && locy <= 0.5)
//			this.maxDist = Tools.distance(locx, locy, 1, 1)/Tools.SQRT2;
//		else if (locx <= 0.5 && locy > 0.5)
//			this.maxDist = Tools.distance(locx, locy, 1, 0)/Tools.SQRT2;
//		else if (locx > 0.5 && locy <= 0.5)
//			this.maxDist = Tools.distance(locx, locy, 0, 1)/Tools.SQRT2;
//		else
//			this.maxDist = Tools.distance(locx, locy, 0, 0)/Tools.SQRT2;
//		
//		this.minWeight = getDistWeight(this.maxDist, this.gamma);
//		this.maxWeight = 1;
	}
	
	public double getDocDist(int i)
	{
		if (this.docDist[i] < 0)
		{
			this.docDistCalc++;
			this.docDist[i] = Tools.distance(this.docLocArray[i], this.queryLoc)/Tools.SQRT2;
			assert(this.docDist[i] <= this.maxDist && this.docDist[i] >= 0);
		}
		return this.docDist[i];
	}
	
	public double getQueryDist(int i)
	{
		if (this.queryDist[i] < 0)
		{
			this.queryDistCalc++;
			double minDist = 1;
			double avgDist = 0;
			double L1 = this.sQueryDocMatrix.getRowVector(i).normL1();
			for (Entry<Integer, Double> docEntry : this.sQueryDocMatrix.getRowVector(i))
			{
				int docId = docEntry.getKey();
				double weight = docEntry.getValue();
				double docDist = this.getDocDist(docId);
				if (docDist < minDist)
					minDist = docDist;
				
				avgDist += docDist * weight / L1;
			}
			this.queryDist[i] = avgDist;//minDist;
			//this.queryDist[i] = minDist;
			
			//System.out.println(avgDist);
		}
		return this.queryDist[i];
	}
	
	public double getDocWeight(int i)
	{
		if (this.docWeight[i] < 0)
		{
			this.docWeight[i] = getDistWeight(this.getDocDist(i), this.gamma);
		}
		return this.docWeight[i];
	}
	
	public double getQueryWeight(int i)
	{
		if (this.queryWeight[i] < 0)
			this.queryWeight[i] = getDistWeight(this.getQueryDist(i), this.gamma);
		return this.queryWeight[i];
	}
	
	public static double getDistWeight (double distance, double gamma)
	{
		//double weight = 1.0 - distance;
		double weight = 1.0/ (distance + Tools.EP);
		weight = Math.pow(weight, gamma);
		
		return weight;
	}
	
	public void print()
	{
		for (int i = 0; i < numDoc; ++i)
		{
			if (this.docDist[i] >= 0)
				System.out.printf("doc%d(%.3f, %.3f): %.4f %.4f\n", i, docLocArray[i].getLat(), docLocArray[i].getLong(), this.docDist[i], this.docWeight[i]);
		}
	}
}
