package sugg.tools;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

public class SortedMapTreeSet <K, V> {
	
	public class KeyValuePair implements Comparable<KeyValuePair>{
		public KeyValuePair (K key, V value)
		{
			this.key = key;
			this.value = value;
		}
		
		public K key;
		public V value;

		@Override
		public int compareTo(KeyValuePair other) {
			if (this.key.equals(other.key))
				return 0;
			
			if (comparator.compare(this.value, other.value) < 0)
				return 1;
			else
				return -1;
		}
	}
	
	private TreeSet<KeyValuePair> treeSet;
	private Map<K, KeyValuePair> map;
	private Comparator<V> comparator;

	public SortedMapTreeSet (Comparator<V> comparator)
	{
		this.treeSet = new TreeSet<KeyValuePair>();
		this.map = new HashMap<K, KeyValuePair>();
		this.comparator = comparator;
	}
	
	public void put (K key, V value)
	{
		if (this.map.containsKey(key))
		{
			KeyValuePair keyValuePair = this.map.get(key);
			this.treeSet.remove(keyValuePair);
			keyValuePair.value = value;
			this.treeSet.add(keyValuePair);
		}
		else
		{
			KeyValuePair keyValuePair = new KeyValuePair(key, value);
			this.map.put(key, keyValuePair);
			this.treeSet.add(keyValuePair);
		}
	}
	
	public KeyValuePair poll()
	{
		KeyValuePair top = this.treeSet.pollLast();
		if (top != null)
			this.map.remove(top.key);
		return top;
	}
	
	public V get(K key)
	{
		if (this.map.containsKey(key))
			return this.map.get(key).value;
		else
			return null;
	}
	
	public void print()
	{
		for (KeyValuePair keyValuePair : this.treeSet)
		{
			System.out.printf("%.3f(%d) ", keyValuePair.value, keyValuePair.key);
		}
		System.out.println();
	}
	
	public boolean empty()
	{
		return this.treeSet.isEmpty();
	}
	
	public int size()
	{
		return this.treeSet.size();
	}
}
