package sugg.tools;

import java.util.Map;
import java.util.Map.Entry;

import sugg.matrix.SMatrix;
import sugg.matrix.SVector;

public class VectorWeighter 
{
	public static double weightCount = 0;
	public static double weightCost = 0;
	public static Timer timer = new Timer();
	
	public static SVector sGetQueryDocVector(int queryId, SMatrix sQueryDocMatrix,
			NodeWeighter weighter, double beta, double range,
			Map<Integer, SVector> queryDocVectorMap)
	{
		SVector weightedQueryDocVector = queryDocVectorMap.get(queryId);
		if (weightedQueryDocVector == null)
		{
			weightCount += 1;
			timer.start();
			
			weightedQueryDocVector = new SVector();
			
			double norm = 0;
			SVector sQueryDocVector = sQueryDocMatrix.getRowVector(queryId);

			double minDistWeight = Double.MAX_VALUE;
			double maxDistWeight = -Double.MAX_VALUE;
			for (Entry<Integer, Double> docEntry : sQueryDocVector)
			{
				int docId = docEntry.getKey();
				
				double distWeight = weighter.getDocWeight(docId);
				if (distWeight < minDistWeight)
					minDistWeight = distWeight;
				if (distWeight > maxDistWeight)
					maxDistWeight = distWeight;
			}
			
			for (Entry<Integer, Double> docEntry : sQueryDocVector)
			{
				int docId = docEntry.getKey();
				double textWeight = docEntry.getValue();
				
				double distWeight = weighter.getDocWeight(docId);
				if (minDistWeight < maxDistWeight)
				//	distWeight /= maxDistWeight;
					distWeight = (distWeight - minDistWeight) / (maxDistWeight - minDistWeight);
				
				double weight = getWeight(textWeight, distWeight, beta);

				if (weighter.getDocDist(docId) <= range)
					weightedQueryDocVector.set(docId, weight);
				norm += weight;
			}
			
			for (Entry<Integer, Double> e : weightedQueryDocVector)
				e.setValue(e.getValue() / norm);
			queryDocVectorMap.put(queryId, weightedQueryDocVector);
			
			weightCost += timer.stop();
		}
		return weightedQueryDocVector;
	}
	
	public static SVector sGetDocQueryVector(int docId, SMatrix sDocQueryMatrix,
			NodeWeighter weighter, double beta, double range,
			Map<Integer, SVector> docQueryVectorMap)
	{
		SVector docQueryVector = docQueryVectorMap.get(docId);
		if (docQueryVector == null)
		{
			weightCount += 1;
			timer.start();
			
			docQueryVector = new SVector();
			
			double minDistWeight = Double.MAX_VALUE;
			double maxDistWeight = -Double.MAX_VALUE;
			SVector sDocQueryVector = sDocQueryMatrix.getRowVector(docId);
			for (Entry<Integer, Double> queryEntry : sDocQueryVector)
			{
				int qId = queryEntry.getKey();
				double distWeight = weighter.getQueryWeight(qId);
				if (distWeight < minDistWeight)
					minDistWeight = distWeight;
				if (distWeight > maxDistWeight)
					maxDistWeight = distWeight;
			}
			
			double norm = 0;
			for (Entry<Integer, Double> queryEntry : sDocQueryVector)
			{
				int qId = queryEntry.getKey();
				double textWeight = queryEntry.getValue();
				

				double distWeight = weighter.getQueryWeight(qId);
				if (minDistWeight < maxDistWeight)
				//	distWeight /= maxDistWeight;
					distWeight = (distWeight - minDistWeight) / (maxDistWeight - minDistWeight);
				
				double weight = getWeight(textWeight, distWeight, beta);
				
				if (weighter.getQueryDist(qId) <= range)
					docQueryVector.set(qId, weight);
				
				norm += weight;
			}
			
			for (Entry<Integer, Double> e : docQueryVector)
				e.setValue(e.getValue() / norm);
			
			docQueryVectorMap.put(docId, docQueryVector);
		
			weightCost += timer.stop();
		}
		return docQueryVector;
	}
	
	public static double getWeight (double textWeight, double distWeight, double beta)
	{
		//double result = textWeight * Math.pow(distWeight, 5);
		double result = beta * textWeight  + (1 - beta) * distWeight;
		return result;
	}
}
